#include "/homes/perrott/VppSim/AMS/AMS_Examples/ams_test/cppsim_modules.h"

//////////////// global probe routines ////////////////

void delete_probe_from_global_list(char *output_filename)
{
PROBE_LINKED_LIST *cur_probe;

cur_probe = global_probe_linked_list;

while (cur_probe != NULL)
   {
    if (strcmp(cur_probe->name,output_filename) == 0)
       break;
    cur_probe = cur_probe->next;
   }
if (cur_probe != NULL)
   {
    if (cur_probe->probe != NULL)
      {
       delete cur_probe->probe;
       cur_probe->probe = NULL;
      }
   }
}

PROBE_LINKED_LIST *retrieve_probe_from_global_list(char *output_filename)
{
PROBE_LINKED_LIST *cur_probe,*prev_probe;

cur_probe = global_probe_linked_list;

if (global_probe_linked_list == NULL)
   {
    global_probe_linked_list = init_probe_linked_list_entry(output_filename);
    cur_probe = global_probe_linked_list;
   }
else
   {
    cur_probe = global_probe_linked_list;
    while(cur_probe != NULL)
      {
	if (strcmp(cur_probe->name,output_filename) == 0)
	   break;
	prev_probe = cur_probe;
	cur_probe = cur_probe->next;
      }
    if (cur_probe == NULL)
      {
        prev_probe->next = init_probe_linked_list_entry(output_filename);
        cur_probe = prev_probe->next;
      }
   }
return(cur_probe);
}

PROBE_LINKED_LIST *init_probe_linked_list_entry(char *output_filename)
{
PROBE_LINKED_LIST *A;
if ((A = (PROBE_LINKED_LIST *) malloc(sizeof(PROBE_LINKED_LIST))) == NULL)
   {
   printf("error in 'init_probe_linked_list_entry':  malloc call failed\n");
   printf("out of memory!\n");
   exit(1);
   }
strcpy(A->name,output_filename);
A->cur_step = 0;
A->cur_step_flag = -1;
A->first_step = -1;
A->last_step = -1;
A->enable = 1;
A->trigger_val = -1.0;
A->prev_trig_sig = -1.0;
A->probe = new Probe(output_filename,Ts);
A->next = NULL;
return(A);
}

PROBE_ELEM *init_probe_elem(char *node_name,PROBE_LINKED_LIST *Probe)
{
int i;
PROBE_ELEM *A;

if ((A = (PROBE_ELEM *) malloc(sizeof(PROBE_ELEM))) == NULL)
   {
   printf("error in 'init_probe_elem':  malloc call failed\n");
   printf("out of memory!\n");
   exit(1);
   }

for (i = 0; node_name[i] != '\0'; i++)
  {
   if (node_name[i] == '(')
      break;
   A->node_name[i] = node_name[i];
  }
A->node_name[i] = '\0';

if (node_name[i] == '(')
   {
    A->is_vector_flag = 1;
    A->max_elem = determine_max_min_elem(node_name,"max");
    A->min_elem = determine_max_min_elem(node_name,"min");
   }
else
   {
    A->is_vector_flag = 0;
    A->max_elem = 0;
    A->min_elem = 0;
   }

A->Probe = Probe;
A->vec = NULL;
A->ivec = NULL;
A->prev_val = 0.0;
return(A);
}

int determine_max_min_elem(char *in, char *control_string)
{
int i,j,flag;
char arg1[MAX_NODE_NAME_LENGTH],arg2[MAX_NODE_NAME_LENGTH];
int arg1_val, arg2_val;

flag = 0;
for (i = 0, j = 0; in[i] != '\0'; i++)
   {
    if (in[i] == '[' || in[i] == '(')
       flag = 1;
    else if (flag == 1 && in[i] != ':' && in[i] != ' ')
      {
       arg1[j++] = in[i];
       arg1[j] = '\0';
      }
    else if (in[i] == ':')
      {
	if (flag == 0)
	  {
          printf("error in 'determine_max_min_elem': did not find initial '[' or '(' character\n");
          printf("  expression reads:  '%s'\n",in);
          exit(1);
	  }
        else
	  {
          flag = 2;
          j = 0;
	  }
      }
    else if (flag == 2 && in[i] != ']' && in[i] != ')' && in[i] != ' ')
      {
       arg2[j++] = in[i];
       arg2[j] = '\0';
      }
    else if (in[i] == ']' || in[i] == ')')
      {
       if (flag == 2)
          flag = 3;
       arg2[j] = '\0';
       break;
      }
   }
if (flag == 0)
   {
     printf("error in 'determine_max_min_elem': did not find initial '[' or '(' character\n");
     printf("  expression reads:  '%s'\n",in);
     exit(1);
   }
else if (flag == 1)
   {
     strcpy(arg2,arg1);
   }
else if (flag == 2)
   {
     printf("error in 'determine_max_min_elem': did not find ']' or ')' character\n");
     printf("  expression reads:  '%s'\n",in);
     exit(1);
   }

if (arg1[0] == '\0' || arg2[0] == '\0')
   return(-1);

arg1_val = atoi(arg1);
arg2_val = atoi(arg2);

if (strcmp(control_string,"max") == 0)
  {
   if (arg1_val > arg2_val)
      return(arg1_val);
   else
      return(arg2_val);
  }
else // min
  {
   if (arg1_val < arg2_val)
      return(arg1_val);
   else
      return(arg2_val);
  }
}

//////////////// global vector routines ////////////////

vector_struct *retrieve_vector_from_global_list(int vec_num)
{
int count;
VECTOR_LINKED_LIST *cur_vector;

cur_vector = global_vector_linked_list;
for (count = 0; count < vec_num; count++)
   {
    if (cur_vector == NULL)
      {
	printf("error in 'retrieve_vector_from_global_list': cur_vector is NULL!\n");
	exit(1);
      }
    cur_vector = cur_vector->next;
   }

if (cur_vector == NULL)
  {
    printf("error in 'retrieve_vector_from_global_list': cur_vector is NULL!\n");
    exit(1);
   }
if (cur_vector->vec == NULL)
   {
    printf("error in 'retrieve_vector_from_global_list': cur_vector->vec is NULL!\n");
    if (cur_vector->ivec != NULL)
      {
        printf("   -> you are trying to access an IntVector with a Vector routine!\n");
      }
        printf("      vector: '%s' (originating module: '%s')\n",
                      cur_vector->ivec->name,cur_vector->ivec->module_name);
    exit(1);
   }
return(cur_vector->vec->vec);
}

int_vector_struct *int_retrieve_vector_from_global_list(int vec_num)
{
int count;
VECTOR_LINKED_LIST *cur_vector;

cur_vector = global_vector_linked_list;
for (count = 0; count < vec_num; count++)
   {
    if (cur_vector == NULL)
      {
	printf("error in 'int_retrieve_vector_from_global_list': cur_vector is NULL!\n");
	exit(1);
      }
    cur_vector = cur_vector->next;
  }
if (cur_vector == NULL)
  {
    printf("error in 'int_retrieve_vector_from_global_list': cur_vector is NULL!\n");
    exit(1);
  }
if (cur_vector->ivec == NULL)
  {
    printf("error in 'int_retrieve_vector_from_global_list': cur_vector->ivec is NULL!\n");
    if (cur_vector->vec != NULL)
      {
        printf("   -> you are trying to access a Vector with an IntVector routine!\n");
      }
        printf("      vector: '%s' (originating module: '%s')\n",
                      cur_vector->vec->name,cur_vector->vec->module_name);
    exit(1);
  }
return(cur_vector->ivec->vec);
}

int add_to_global_vector_list(Vector *in)
{
int count;
VECTOR_LINKED_LIST *cur_vector;

if (global_vector_linked_list == NULL)
   {
    count = 0;
    global_vector_linked_list = init_vector_linked_list_entry(in);
   }
else
   {
    count = 1;
    cur_vector = global_vector_linked_list;
    while(cur_vector->next != NULL)
      {
        count++;
	cur_vector = cur_vector->next;
      }
    cur_vector->next = init_vector_linked_list_entry(in);
   }
return(count);
}

int add_to_global_vector_list(IntVector *in)
{
int count;
VECTOR_LINKED_LIST *cur_vector;

if (global_vector_linked_list == NULL)
   {
    count = 0;
    global_vector_linked_list = init_vector_linked_list_entry(in);
   }
else
   {
    count = 1;
    cur_vector = global_vector_linked_list;
    while(cur_vector->next != NULL)
      {
        count++;
	cur_vector = cur_vector->next;
      }
    cur_vector->next = init_vector_linked_list_entry(in);
   }
return(count);
}

VECTOR_LINKED_LIST *init_vector_linked_list_entry(Vector *in)
{
VECTOR_LINKED_LIST *A;

if ((A = (VECTOR_LINKED_LIST *) malloc(sizeof(VECTOR_LINKED_LIST))) == NULL)
   {
   printf("error in 'init_vector_linked_list_entry':  malloc call failed\n");
   printf("out of memory!\n");
   exit(1);
   }
A->vec = in;
A->ivec = NULL;
A->next = NULL;
return(A);
}

VECTOR_LINKED_LIST *init_vector_linked_list_entry(IntVector *in)
{
VECTOR_LINKED_LIST *A;

if ((A = (VECTOR_LINKED_LIST *) malloc(sizeof(VECTOR_LINKED_LIST))) == NULL)
   {
   printf("error in 'init_vector_linked_list_entry':  malloc call failed\n");
   printf("out of memory!\n");
   exit(1);
   }
A->vec = NULL;
A->ivec = in;
A->next = NULL;
return(A);
}

//////////////// set Ts value routine ////////////////

extern "C" int set_Ts_value_cpp_calltf()
{
Ts = tf_getrealp(1);
}

extern "C" int set_Ts_value_cpp_misctf(int data, int reason)
{
switch(reason)
   {
   case reason_endofcompile:
      Ts = tf_getrealp(1);
      break;
   }
}

//////////////// probe matlab routines ////////////////

extern "C" int probe_start_sample_cpp_calltf()
{
PROBE_LINKED_LIST *Probe;
double val;
int int_val;

Probe = (PROBE_LINKED_LIST *) tf_getworkarea();

if (Probe->cur_step_flag == 0)
    Probe->cur_step++;

if (tf_typep(2) == tf_readonlyreal || tf_typep(2) == tf_readwritereal)
  {
   val = tf_getrealp(2);
   Probe->first_step = (int) floor(val+0.5);
  }
else
  {
   int_val = tf_getp(2);
   Probe->first_step = int_val;
  }
}

extern "C" int probe_start_sample_cpp_misctf(int data, int reason)
{
char *output_filename;
PROBE_LINKED_LIST *Probe;

switch(reason)
   {
   case reason_endofcompile:
      if (tf_nump() != 2)
         {
          printf("ERROR: $probe_start_sample requires 2 arguments!\n");
          printf("   -> in this case, %d arguments were specified\n",tf_nump());
          exit(1);
         }
      if (tf_typep(1) != tf_string)
         {
          printf("ERROR: $probe_start_sample requires first argument to be a string!\n");
          exit(1);
         }
      if (tf_typep(2) == tf_string)
         {
          printf("ERROR: $probe_start_sample requires second argument to not be a string!\n");
          exit(1);
         }
      output_filename = tf_getcstringp(1);
      Probe = retrieve_probe_from_global_list(output_filename);
      if (Probe->cur_step_flag < 0)
          Probe->cur_step_flag = 0;
      tf_setworkarea((PLI_BYTE8 *) Probe);
   break;
   }
}

extern "C" int probe_end_sample_cpp_calltf()
{
PROBE_LINKED_LIST *Probe;
double val;
int int_val;

Probe = (PROBE_LINKED_LIST *) tf_getworkarea();

if (Probe->cur_step_flag == 1)
    Probe->cur_step++;

if (tf_typep(2) == tf_readonlyreal || tf_typep(2) == tf_readwritereal)
  {
   val = tf_getrealp(2);
   Probe->last_step = (int) floor(val+0.5);
  }
else
  {
   int_val = tf_getp(2);
   Probe->last_step = int_val;
  }
}

extern "C" int probe_end_sample_cpp_misctf(int data, int reason)
{
char *output_filename;
PROBE_LINKED_LIST *Probe;

switch(reason)
   {
   case reason_endofcompile:
      if (tf_nump() != 2)
         {
          printf("ERROR: $probe_end_sample requires 2 arguments!\n");
          printf("   -> in this case, %d arguments were specified\n",tf_nump());
          exit(1);
         }
      if (tf_typep(1) != tf_string)
         {
          printf("ERROR: $probe_end_sample requires first argument to be a string!\n");
          exit(1);
         }
      if (tf_typep(2) == tf_string)
         {
          printf("ERROR: $probe_end_sample requires second argument to not be a string!\n");
          exit(1);
         }
      output_filename = tf_getcstringp(1);
      Probe = retrieve_probe_from_global_list(output_filename);
      if (Probe->cur_step_flag < 0)
          Probe->cur_step_flag = 1;
      tf_setworkarea((PLI_BYTE8 *) Probe);
   break;
   }
}

extern "C" int probe_start_time_cpp_calltf()
{
PROBE_LINKED_LIST *Probe;
double val;
int int_val;

Probe = (PROBE_LINKED_LIST *) tf_getworkarea();

if (Probe->cur_step_flag == 2)
    Probe->cur_step++;

if (tf_typep(2) == tf_readonlyreal || tf_typep(2) == tf_readwritereal)
  {
   val = tf_getrealp(2);
   Probe->first_step = (int) floor(val/Ts+0.5);
  }
else
  {
   int_val = tf_getp(2);
   Probe->first_step = (int) (((double) int_val)/Ts);
  }
}

extern "C" int probe_start_time_cpp_misctf(int data, int reason)
{
char *output_filename;
PROBE_LINKED_LIST *Probe;

switch(reason)
   {
   case reason_endofcompile:
      if (tf_nump() != 2)
         {
          printf("ERROR: $probe_start_time requires 2 arguments!\n");
          printf("   -> in this case, %d arguments were specified\n",tf_nump());
          exit(1);
         }
      if (tf_typep(1) != tf_string)
         {
          printf("ERROR: $probe_start_time requires first argument to be a string!\n");
          exit(1);
         }
      if (tf_typep(2) == tf_string)
         {
          printf("ERROR: $probe_start_time requires second argument to not be a string!\n");
          exit(1);
         }
      output_filename = tf_getcstringp(1);
      Probe = retrieve_probe_from_global_list(output_filename);
      if (Probe->cur_step_flag < 0)
          Probe->cur_step_flag = 2;
      tf_setworkarea((PLI_BYTE8 *) Probe);
   break;
   }
}

extern "C" int probe_end_time_cpp_calltf()
{
PROBE_LINKED_LIST *Probe;
double val;
int int_val;

Probe = (PROBE_LINKED_LIST *) tf_getworkarea();

if (Probe->cur_step_flag == 3)
    Probe->cur_step++;

if (tf_typep(2) == tf_readonlyreal || tf_typep(2) == tf_readwritereal)
  {
   val = tf_getrealp(2);
   Probe->last_step = (int) floor(val/Ts+0.5);
  }
else
  {
   int_val = tf_getp(2);
   Probe->last_step = (int) (((double) int_val)/Ts);
  }
}

extern "C" int probe_end_time_cpp_misctf(int data, int reason)
{
char *output_filename;
PROBE_LINKED_LIST *Probe;

switch(reason)
   {
   case reason_endofcompile:
      if (tf_nump() != 2)
         {
          printf("ERROR: $probe_end_time requires 2 arguments!\n");
          printf("   -> in this case, %d arguments were specified\n",tf_nump());
          exit(1);
         }
      if (tf_typep(1) != tf_string)
         {
          printf("ERROR: $probe_end_time requires first argument to be a string!\n");
          exit(1);
         }
      if (tf_typep(2) == tf_string)
         {
          printf("ERROR: $probe_end_time requires second argument to not be a string!\n");
          exit(1);
         }
      output_filename = tf_getcstringp(1);
      Probe = retrieve_probe_from_global_list(output_filename);
      if (Probe->cur_step_flag < 0)
          Probe->cur_step_flag = 3;
      tf_setworkarea((PLI_BYTE8 *) Probe);
   break;
   }
}

extern "C" int probe_enable_cpp_calltf()
{
PROBE_LINKED_LIST *Probe;
double val;
int int_val;

Probe = (PROBE_LINKED_LIST *) tf_getworkarea();

if (tf_typep(2) == tf_readonlyreal || tf_typep(2) == tf_readwritereal)
  {
   val = tf_getrealp(2);
   Probe->enable = val > 0.0 ? 1 : 0;
  }
else
  {
   int_val = tf_getp(2);
   Probe->enable = int_val > 0 ? 1 : 0;
  }
}

extern "C" int probe_enable_cpp_misctf(int data, int reason)
{
char *output_filename;
PROBE_LINKED_LIST *Probe;

switch(reason)
   {
   case reason_endofcompile:
      if (tf_nump() != 2)
         {
          printf("ERROR: $probe_enable requires 2 arguments!\n");
          printf("   -> in this case, %d arguments were specified\n",tf_nump());
          exit(1);
         }
      if (tf_typep(1) != tf_string)
         {
          printf("ERROR: $probe_enable requires first argument to be a string!\n");
          exit(1);
         }
      if (tf_typep(2) == tf_string)
         {
          printf("ERROR: $probe_enable requires second argument to not be a string!\n");
          exit(1);
         }
      output_filename = tf_getcstringp(1);
      Probe = retrieve_probe_from_global_list(output_filename);
      tf_setworkarea((PLI_BYTE8 *) Probe);
   break;
   }
}

extern "C" int probe_trigger_cpp_calltf()
{
PROBE_LINKED_LIST *Probe;
double cur_val, prev_val;
Probe = (PROBE_LINKED_LIST *) tf_getworkarea();

cur_val = tf_getrealp(2);
prev_val = Probe->prev_trig_sig;
if (prev_val == -1.0 && cur_val != -1.0)
   Probe->trigger_val = cur_val;
else
   Probe->trigger_val = -2.0;
Probe->prev_trig_sig = cur_val;
}

extern "C" int probe_trigger_cpp_misctf(int data, int reason)
{
char *output_filename;
PROBE_LINKED_LIST *Probe;

switch(reason)
   {
   case reason_endofcompile:
      if (tf_nump() != 2)
         {
          printf("ERROR: $probe_trigger requires 2 arguments!\n");
          printf("   -> in this case, %d arguments were specified\n",tf_nump());
          exit(1);
         }
      if (tf_typep(1) != tf_string)
         {
          printf("ERROR: $probe_trigger requires first argument to be a string!\n");
          exit(1);
         }
      if (tf_typep(2) != tf_readonlyreal && tf_typep(2) != tf_readwritereal)
         {
          printf("ERROR: $probe_trigger requires second argument to be a real number!\n");
          exit(1);
         }
      output_filename = tf_getcstringp(1);
      Probe = retrieve_probe_from_global_list(output_filename);
      tf_setworkarea((PLI_BYTE8 *) Probe);
   break;
   }
}

extern "C" int probe_node_cpp_calltf()
{
PROBE_ELEM *probe_elem;
VECTOR_LINKED_LIST *cur_vector;
char *node_name;
char vec_node_name[MAX_NODE_NAME_LENGTH];
double cur_val, prev_val, interp, interp_val;
int int_val;
int i;

node_name = tf_getcstringp(2);
probe_elem = (PROBE_ELEM *) tf_getworkarea();

if ((probe_elem->Probe->cur_step < probe_elem->Probe->first_step &&
     probe_elem->Probe->first_step != -1) || 
    (probe_elem->Probe->cur_step > probe_elem->Probe->last_step &&
     probe_elem->Probe->last_step != -1) || 
     probe_elem->Probe->enable == 0 || 
     probe_elem->Probe->trigger_val < -1.5)
   return(0);

if (probe_elem->is_vector_flag == 1)
  {
   int_val = tf_getp(3);
   if (probe_elem->vec == NULL && probe_elem->ivec == NULL)
     {
      cur_vector = global_vector_linked_list;
      for (i = 0; i < int_val; i++)
        {
         if (cur_vector == NULL)
           {
	    printf("error in 'probe_node_cpp_calltf': cur_vector is NULL!\n");
	    exit(1);
           }
         if (cur_vector->next == NULL)
           {
	    printf("error in 'probe_node_cpp_calltf': cur_vector->next is NULL!\n");
	    exit(1);
           }
         cur_vector = cur_vector->next;
        }
      if (cur_vector->vec != NULL)
	 probe_elem->vec = cur_vector->vec;
      else if (cur_vector->ivec != NULL)
	 probe_elem->ivec = cur_vector->ivec;
      else
	{
	 printf("error in 'probe_node_cpp_calltf': vec and ivec are NULL!\n");
         exit(1);
	}
      if (probe_elem->min_elem < 0 || probe_elem->max_elem < 0)
	{
	  probe_elem->min_elem = 0;
          if (cur_vector->vec != NULL)
	     probe_elem->max_elem = cur_vector->vec->get_length()-1;
          else
	     probe_elem->max_elem = cur_vector->ivec->get_length()-1;
	}
     }
   if (probe_elem->vec != NULL)
     {
      for (i = probe_elem->min_elem; i <= probe_elem->max_elem; i++)
	{
	 sprintf(vec_node_name,"%s_%d",probe_elem->node_name,i);
         probe_elem->Probe->probe->inp(probe_elem->vec->get_elem(i),vec_node_name);
	}
     }
   else
     {
      for (i = probe_elem->min_elem; i <= probe_elem->max_elem; i++)
	{
	 sprintf(vec_node_name,"%s_%d",probe_elem->node_name,i);
         probe_elem->Probe->probe->inp(probe_elem->ivec->get_elem(i),vec_node_name);
	}
     }
  }
else if (tf_typep(3) == tf_readonlyreal || tf_typep(3) == tf_readwritereal)
  {
   cur_val = tf_getrealp(3);
   prev_val = probe_elem->prev_val;
   interp = probe_elem->Probe->trigger_val;
   interp_val = (prev_val*(interp+1.0)+cur_val*(-interp+1.0))/2.0;
   probe_elem->Probe->probe->inp(interp_val,node_name);
   probe_elem->prev_val = cur_val;
  }
else
  {
   int_val = tf_getp(3);
   probe_elem->Probe->probe->inp(int_val,node_name);
  }
}

extern "C" int probe_node_cpp_misctf(int data, int reason)
{
char *output_filename, *node_name;
PROBE_LINKED_LIST *Probe;
PROBE_ELEM *probe_elem;

switch(reason)
   {
   case reason_endofcompile:
      if (tf_nump() != 3)
         {
          printf("ERROR: $probe_node requires 3 arguments!\n");
          printf("   -> in this case, %d arguments were specified\n",tf_nump());
          exit(1);
         }
      if (tf_typep(1) != tf_string)
         {
          printf("ERROR: $probe_node requires first argument to be a string!\n");
          exit(1);
         }
      if (tf_typep(2) != tf_string)
         {
          printf("ERROR: $probe_node requires second argument to be a string!\n");
          exit(1);
         }
      if (tf_typep(3) == tf_string)
         {
          printf("ERROR: $probe_node requires third argument to not be a string!\n");
          exit(1);
         }
      output_filename = tf_getcstringp(1);
      node_name = tf_getcstringp(2);
      Probe = retrieve_probe_from_global_list(output_filename);
      probe_elem = init_probe_elem(node_name,Probe);
      tf_setworkarea((PLI_BYTE8 *) probe_elem);
   break;
   case reason_finish:
      // destroy C++ probe object
      output_filename = tf_getcstringp(1);
      delete_probe_from_global_list(output_filename);
      probe_elem = (PROBE_ELEM *) tf_getworkarea();
      delete probe_elem;
   break;
   }
}

//////////////// access Vector element routine ////////////////

extern "C" int get_vector_elem_cpp_calltf()
{
int elem_num;
vector_struct *vec_hold;
int vec_length;

vec_hold = (vector_struct *) tf_getworkarea();
if (vec_hold == NULL)
   {
   vec_hold = retrieve_vector_from_global_list(tf_getp(1));
   tf_setworkarea((PLI_BYTE8 *) vec_hold);
   tf_putrealp(3,0.0);
   }
else
   {
   elem_num = tf_getp(2);

   vec_length = vec_hold->length;

   if (elem_num < 0 || elem_num >= vec_length)
      {
      printf("ERROR:  tried to access a Vector element\n");
      printf("    using $get_vector_elem and went beyond the available range\n");
      printf("    vector: '%s' (originating module: '%s')\n",vec_hold->name,vec_hold->module_name);
      printf("    -> desired elem_num = %d, vector_length = %d\n",elem_num,vec_length);
      exit(1);
      }
   tf_putrealp(3,vec_hold->elem[elem_num]);
   }
}

extern "C" int get_vector_elem_cpp_misctf(int data, int reason)
{
switch(reason)
   {
   case reason_endofcompile:
      if (tf_nump() != 3)
         {
          printf("ERROR: $get_vector_elem requires 3 arguments!\n");
          printf("   -> in this, %d arguments were specified\n",tf_nump());
          exit(1);
         }
      tf_setworkarea((PLI_BYTE8 *) NULL);
      break;
   }
}

//////////////// access IntVector element routine ////////////////

extern "C" int get_intvector_elem_cpp_calltf()
{
int elem_num;
int_vector_struct *vec_hold;
int vec_length;

vec_hold = (int_vector_struct *) tf_getworkarea();
if (vec_hold == NULL)
   {
   vec_hold = int_retrieve_vector_from_global_list(tf_getp(1));
   tf_setworkarea((PLI_BYTE8 *) vec_hold);
   tf_putp(3,0);
   }
else
   {
   elem_num = tf_getp(2);

   vec_length = vec_hold->length;

   if (elem_num < 0 || elem_num >= vec_length)
      {
      printf("ERROR:  tried to access a IntVector element\n");
      printf("    using $get_intvector_elem and went beyond the available range\n");
      printf("    vector: '%s' (originating module: '%s')\n",vec_hold->name,vec_hold->module_name);
      printf("    -> desired elem_num = %d, vector_length = %d\n",elem_num,vec_length);
      exit(1);
      }
   tf_putp(3,vec_hold->elem[elem_num]);
   }
}

extern "C" int get_intvector_elem_cpp_misctf(int data, int reason)
{
switch(reason)
   {
   case reason_endofcompile:
      if (tf_nump() != 3)
         {
          printf("ERROR: $get_intvector_elem requires 3 arguments!\n");
          printf("   -> in this, %d arguments were specified\n",tf_nump());
          exit(1);
         }
      tf_setworkarea((PLI_BYTE8 *) NULL);
      break;
   }
}

/////////// reset cppsim modules routine ////////////

extern "C" int reset_cppsim_modules_cpp_calltf()
{
global_cppsim_mode_flag = 0;
}

extern "C" int reset_cppsim_modules_cpp_misctf(int data, int reason)
{
switch(reason)
   {
   case reason_endofcompile:
      break;
   }
}

/////////// update cppsim mode routine ////////////

extern "C" int update_cppsim_mode_cpp_calltf()
{
if (global_cppsim_mode_flag != 1)
   global_cppsim_mode_flag = 1;
}

extern "C" int update_cppsim_mode_cpp_misctf(int data, int reason)
{
switch(reason)
   {
   case reason_endofcompile:
      break;
   }
}

/////////////////////////////////////////////////////

lpf_first_order_cpp::lpf_first_order_cpp(double gain,double fp)
  :  filt("K","1 + 1/(2*pi*fp)*s","K,fp,Ts",gain,fp,Ts)
{
// Automatically generated initialization code
gain_arg_in = gain;
fp_arg_in = fp;
out = 0.0;
}

lpf_first_order_cpp::~lpf_first_order_cpp()
{
// Parameters
double gain = gain_arg_in;
double fp = fp_arg_in;

// User supplied code from the 'end:' section
}

void lpf_first_order_cpp::inp(double in_arg_in)
{
// Inputs
in = in_arg_in;

// Parameters
double gain = gain_arg_in;
double fp = fp_arg_in;

// User supplied code from the 'code:' section
filt.inp(in);
out = filt.out;
}

void lpf_first_order_cpp::init()
{
// Inputs

// Parameters
double gain = gain_arg_in;
double fp = fp_arg_in;

// User supplied code from the 'init:' section
}

void lpf_first_order_cpp::init_output_vectors()
{
// Inputs

// Parameters
double gain = gain_arg_in;
double fp = fp_arg_in;

// Set output vector lengths according to 'set_output_vector_lengths:' section

// Freeze the length of all output vectors
}

extern "C" int lpf_first_order_cpp_calltf()
{
lpf_first_order_cpp *lpf_first_order_obj;

double in;

// retrieve input values
in = tf_getrealp(1);

// retrieve C++ object and execute it
lpf_first_order_obj = (lpf_first_order_cpp *) tf_getworkarea();

if (global_cppsim_mode_flag == 0)
   lpf_first_order_obj->init();
else
   lpf_first_order_obj->inp(in);

// return output values
tf_putrealp(2, lpf_first_order_obj->out);

}

extern "C" int lpf_first_order_cpp_misctf(int data, int reason)
{
lpf_first_order_cpp *lpf_first_order_obj;

double gain;
double fp;

switch(reason)
   {
   case reason_endofcompile:
      // retrieve parameter values
      gain = tf_getrealp(3);
      fp = tf_getrealp(4);

      // create C++ object
      lpf_first_order_obj = new lpf_first_order_cpp(gain, fp);
      tf_setworkarea((PLI_BYTE8 *) lpf_first_order_obj);

      // initialize and return output vectors
      lpf_first_order_obj->init_output_vectors();
      lpf_first_order_obj->init();

   break;
   case reason_finish:
      // destroy C++ object
      lpf_first_order_obj = (lpf_first_order_cpp *) tf_getworkarea();
      delete lpf_first_order_obj;
   break;
   }
}

/////////////////////////////////////////////////////

vco_cpp::vco_cpp(double freq,double kvco)
  :  vco("fc + Kv*x","fc,Kv,Ts",freq,kvco,Ts)
{
// Automatically generated initialization code
freq_arg_in = freq;
kvco_arg_in = kvco;
sineout = 0.0;
squareout = 0.0;
}

vco_cpp::~vco_cpp()
{
// Parameters
double freq = freq_arg_in;
double kvco = kvco_arg_in;

// User supplied code from the 'end:' section
}

void vco_cpp::inp(double vctrl_arg_in)
{
// Inputs
vctrl = vctrl_arg_in;

// Parameters
double freq = freq_arg_in;
double kvco = kvco_arg_in;

// User supplied code from the 'code:' section
vco.inp(vctrl);
squareout = vco.out;
sineout = sin(vco.phase);
}

void vco_cpp::init()
{
// Inputs

// Parameters
double freq = freq_arg_in;
double kvco = kvco_arg_in;

// User supplied code from the 'init:' section
}

void vco_cpp::init_output_vectors()
{
// Inputs

// Parameters
double freq = freq_arg_in;
double kvco = kvco_arg_in;

// Set output vector lengths according to 'set_output_vector_lengths:' section

// Freeze the length of all output vectors
}

extern "C" int vco_cpp_calltf()
{
vco_cpp *vco_obj;

double vctrl;

// retrieve input values
vctrl = tf_getrealp(1);

// retrieve C++ object and execute it
vco_obj = (vco_cpp *) tf_getworkarea();

if (global_cppsim_mode_flag == 0)
   vco_obj->init();
else
   vco_obj->inp(vctrl);

// return output values
tf_putrealp(2, vco_obj->sineout);
tf_putrealp(3, vco_obj->squareout);

}

extern "C" int vco_cpp_misctf(int data, int reason)
{
vco_cpp *vco_obj;

double freq;
double kvco;

switch(reason)
   {
   case reason_endofcompile:
      // retrieve parameter values
      freq = tf_getrealp(4);
      kvco = tf_getrealp(5);

      // create C++ object
      vco_obj = new vco_cpp(freq, kvco);
      tf_setworkarea((PLI_BYTE8 *) vco_obj);

      // initialize and return output vectors
      vco_obj->init_output_vectors();
      vco_obj->init();

   break;
   case reason_finish:
      // destroy C++ object
      vco_obj = (vco_cpp *) tf_getworkarea();
      delete vco_obj;
   break;
   }
}

/////////////////////////////////////////////////////

