%%% Test bench for High-Speed Link Transceiver Example %%%
clear all; close all; clc;
set(0,  'DefaultAxesFontSize', 15,...
    'DefaultTextfontSize', 15,...
    'DefaultLineLineWidth', 2);

% Load HspiceToolBox
addpath('/home/vault_stojanovic/software/VppSimShared/HspiceToolbox');  % Load HspiceToolBox

Tsym = 1/8e9;
ratio = 20;
Ts=Tsym/ratio;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Part1: Read and plot the s4p file
if 0
    channelName='channel_data.s4p';
    mode='s21';
    [f,H]=extract_mode_from_s4p(channelName,mode);
    figure(1)
    subplot(211),plot(f*1e-9,20*log10(abs(H)),'b'); xlabel('freqyency [GHz]'); ylabel('Transfer function [dB]'); grid on;
    ylim([-100, 5]);
    title('Channel Transfer Function');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Part2: Generate and plot the channel impulse reponse
if 0
    imp=xfr_fn_to_imp(f,H,Ts,Tsym); %%% Ts sampled impulse response
    nsym_short=300; %%% persistance of the impulse response tail in the channel in terms of the number of symbols
    imp_short=imp(1:min(length(imp), floor(nsym_short*Tsym/Ts)));
    pr = conv(imp_short, ones(1, floor(Tsym/Ts)));
    subplot(212), plot((1: length(pr))*Ts, pr,'b.-'); hold on; grid on;
    ylim([-0.2, 0.5]);
    title('Pulse Response');
    save link_channel.dat imp_short -ascii;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Part3: Compute initial ZFE equalizer solution
if 0
    eq_tap_length=5;    % length of the pre-emphasis filter
    eq_tap_pre=2;       % length of taps for post-cursor
    eq_taps=zfe(imp_short,Tsym,Ts,eq_tap_length,eq_tap_pre)
    % eq_taps=[0 1 0 0 0];
    save eq_taps.dat eq_taps -ascii;
end

%%% Run the simulation
%   cppsim test.par;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Part4: Eye diagram drawing
%%% Load the signals after simulation
x=loadsig('test.tr0');
lssig(x)
if 1
    figure(3)
    eyesig(x, 2*Tsym, 100*Tsym,'ch_out');
end

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %% Part4.1: Correlation of the Tx data and Rx data
% %%% Load the signals after simulation
% t=evalsig(x,'TIME');
% ber = evalsig(x, 'ber');
% tx_data_out=evalsig(x,'tx_data_out');
% tx_data_out = tx_data_out(100: floor(Tsym/Ts): end);
% rx_data_out=evalsig(x,'rx_data_out');
% rx_data_out = rx_data_out(100: floor(Tsym/Ts): end);  %% you may want to change 5/10 to 10/5.
% figure(4)
% tmp = xcorr(tx_data_out, rx_data_out);
% l = (length(tmp) + 1) /2;
% stem([1: length(tmp)]-l , tmp);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Part5: CDR output phase
if 1
    y = loadsig('phase.tr0');
    lssig(y)
    tp = evalsig(y, 'TIME');
    phase = evalsig(y, 'phase');
    figure(6)
    stem(tp, phase);
    xlabel('# of adaptation');  ylabel('CDR output phase');
    ylim([0, 1.2*ratio])
    grid on;
end
