////////// Verilog Modules from Cadence Directories /////////

module vpp_internal_int_to_double_convert(in, out, update_in, update_out);

   input [31:0] in;
   input update_in;
   output out;
   output update_out;

   wreal out;

   integer in_iv;
   reg update_out;
   real out_rv;

   assign out = out_rv;
 
   initial
     begin
       out_rv = 0.0;
       assign in_iv = in;
     end

   always@update_in
      begin
        out_rv = in_iv;
        update_out = update_in;
      end

endmodule


module vpp_internal_double_to_int_convert(in, out, update_in, update_out);

   input in;
   input update_in;
   output [31:0] out;
   output update_out;

   wreal in;
   real in_rv;
   reg update_out;
   integer out_iv;

   assign out = out_iv;
 
   initial
     begin
       out_iv = 32'd0;
       assign in_rv = in;
     end

   always@update_in
      begin
        out_iv = in_rv;
        update_out = update_in;
      end

endmodule


module vpp_internal_double_interp_to_int_convert(in, out, update_in, update_out);

   input in;
   input update_in;
   output [31:0] out;
   output update_out;

   wreal in;
   real in_rv;
   reg update_out;
   reg out_bv;

   assign out = {31'd0,out_bv}; 

   initial
     begin
       out_bv = 1'b0;
       assign in_rv = in;
     end

   always@update_in
      begin
         if (out_bv == 1'b0)
            begin
               if (in_rv != -1.0)
                   out_bv = 1'b1;
            end
         else
            begin
               if (in_rv != 1.0)
                   out_bv = 1'b0;
            end
        update_out = update_in;
      end

endmodule



module prbsV(clkV, outV);

input	clkV;
output	outV;

wire	outV;

reg [31:0] lfsr;
reg	tmp;

initial begin
	//set seeds
 	lfsr = 32'b11111111111111111111111111111110;  
end

assign outV = lfsr[0];

always @ (posedge clkV)
begin   
   tmp = ~((lfsr[31]^lfsr[21])^(lfsr[1]^lfsr[0]));
   lfsr = { lfsr[30: 0], tmp};
//   #5
//   $monitor("\n %b %b %b %b %b", lfsr[31], lfsr[21], lfsr[1], lfsr[0], tmp);
end


endmodule




////// End of Verilog Modules from Cadence Directories //////



//////////      Auto-generated Verilog Modules      /////////

/////////////////////////////////////////////////////

/////////////////////////////////////////////////////

/////////////////////////////////////////////////////

/////////////////////////////////////////////////////

module signal_source_cpp(phase, clk, out, update_in, update_out);

   parameter freq = 0.0000000000e+00;
   parameter stype = 0;
   input phase;
   input update_in;
   output clk;
   output out;
   output update_out;

   wreal phase;
   real phase_rv;
   wreal clk;
   real clk_rv;
   wreal out;
   real out_rv;
   reg update_out;

   assign clk = clk_rv;
   assign out = out_rv;

   initial
      begin
        assign phase_rv = phase;
      end

   always@update_in
      begin
        $signal_source_cpp(phase_rv,clk_rv,out_rv,freq,stype);
        update_out = update_in;
      end

endmodule

/////////////////////////////////////////////////////

module tx_eqV_cpp(tx_eq_out, eq_settings_vec, clk_in, data_in, update_in, update_out);

   input [31:0] eq_settings_vec;
   input clk_in;
   input data_in;
   input update_in;
   output tx_eq_out;
   output update_out;

   integer eq_settings_vec_iv;
   wreal clk_in;
   real clk_in_rv;
   wreal data_in;
   real data_in_rv;
   wreal tx_eq_out;
   real tx_eq_out_rv;
   reg update_out;

   assign tx_eq_out = tx_eq_out_rv;

   initial
      begin
        assign eq_settings_vec_iv = eq_settings_vec;
        assign clk_in_rv = clk_in;
        assign data_in_rv = data_in;
      end

   always@update_in
      begin
        $tx_eqV_cpp(eq_settings_vec_iv,clk_in_rv,data_in_rv,tx_eq_out_rv);
        update_out = update_in;
      end

endmodule

/////////////////////////////////////////////////////

module eq_setting_from_file_cpp(vec_out, update_in, update_out);

   parameter num_tap = 0;
   input update_in;
   output [31:0] vec_out;
   output update_out;

   integer vec_out_iv;
   reg update_out;

   assign vec_out = vec_out_iv;

   always@update_in
      begin
        $eq_setting_from_file_cpp(vec_out_iv,num_tap);
        update_out = update_in;
      end

endmodule

/////////////////////////////////////////////////////

module constant_cpp(out, update_in, update_out);

   parameter consval = 0.0000000000e+00;
   input update_in;
   output out;
   output update_out;

   wreal out;
   real out_rv;
   reg update_out;

   assign out = out_rv;

   always@update_in
      begin
        $constant_cpp(out_rv,consval);
        update_out = update_in;
      end

endmodule

/////////////////////////////////////////////////////

module link_channel_cpp(channel_out, channel_in, update_in, update_out);

   input channel_in;
   input update_in;
   output channel_out;
   output update_out;

   wreal channel_in;
   real channel_in_rv;
   wreal channel_out;
   real channel_out_rv;
   reg update_out;

   assign channel_out = channel_out_rv;

   initial
      begin
        assign channel_in_rv = channel_in;
      end

   always@update_in
      begin
        $link_channel_cpp(channel_in_rv,channel_out_rv);
        update_out = update_in;
      end

endmodule

/////////////////////////////////////////////////////

module divider_ideal_cpp(div_val, in, out, update_in, update_out);

   input div_val;
   input in;
   input update_in;
   output out;
   output update_out;

   wreal div_val;
   real div_val_rv;
   wreal in;
   real in_rv;
   wreal out;
   real out_rv;
   reg update_out;

   assign out = out_rv;

   initial
      begin
        assign div_val_rv = div_val;
        assign in_rv = in;
      end

   always@update_in
      begin
        $divider_ideal_cpp(div_val_rv,in_rv,out_rv);
        update_out = update_in;
      end

endmodule

/////////////////////////////////////////////////////

module phase_interpolator_cpp(clk_edge, clk_data, clk_adapt, clk, phase_select, update_in, update_out);

   parameter num_phases = 0;
   parameter clk_freq = 0.0000000000e+00;
   input clk;
   input [31:0] phase_select;
   input update_in;
   output clk_edge;
   output clk_data;
   output clk_adapt;
   output update_out;

   wreal clk;
   real clk_rv;
   integer phase_select_iv;
   wreal clk_edge;
   real clk_edge_rv;
   wreal clk_data;
   real clk_data_rv;
   wreal clk_adapt;
   real clk_adapt_rv;
   reg update_out;

   assign clk_edge = clk_edge_rv;
   assign clk_data = clk_data_rv;
   assign clk_adapt = clk_adapt_rv;

   initial
      begin
        assign clk_rv = clk;
        assign phase_select_iv = phase_select;
      end

   always@update_in
      begin
        $phase_interpolator_cpp(clk_rv,phase_select_iv,clk_edge_rv,clk_data_rv,clk_adapt_rv,num_phases,clk_freq);
        update_out = update_in;
      end

endmodule

/////////////////////////////////////////////////////

module cdr_counter_cpp(clk, enable_b, inc_dec, count, update_in, update_out);

   parameter range = 0;
   input clk;
   input enable_b;
   input inc_dec;
   input update_in;
   output [31:0] count;
   output update_out;

   wreal clk;
   real clk_rv;
   wreal enable_b;
   real enable_b_rv;
   wreal inc_dec;
   real inc_dec_rv;
   integer count_iv;
   reg update_out;

   assign count = count_iv;

   initial
      begin
        assign clk_rv = clk;
        assign enable_b_rv = enable_b;
        assign inc_dec_rv = inc_dec;
      end

   always@update_in
      begin
        $cdr_counter_cpp(clk_rv,enable_b_rv,inc_dec_rv,count_iv,range);
        update_out = update_in;
      end

endmodule

/////////////////////////////////////////////////////

module cdr_maj_vote_cpp(hold, up_down, div_clk, clk, valid_transition, early_late, update_in, update_out);

   parameter num_maj_votes = 0;
   input div_clk;
   input clk;
   input valid_transition;
   input early_late;
   input update_in;
   output hold;
   output up_down;
   output update_out;

   wreal div_clk;
   real div_clk_rv;
   wreal clk;
   real clk_rv;
   wreal valid_transition;
   real valid_transition_rv;
   wreal early_late;
   real early_late_rv;
   wreal hold;
   real hold_rv;
   wreal up_down;
   real up_down_rv;
   reg update_out;

   assign hold = hold_rv;
   assign up_down = up_down_rv;

   initial
      begin
        assign div_clk_rv = div_clk;
        assign clk_rv = clk;
        assign valid_transition_rv = valid_transition;
        assign early_late_rv = early_late;
      end

   always@update_in
      begin
        $cdr_maj_vote_cpp(div_clk_rv,clk_rv,valid_transition_rv,early_late_rv,hold_rv,up_down_rv,num_maj_votes);
        update_out = update_in;
      end

endmodule

/////////////////////////////////////////////////////

module xor2_cpp(y, a, b, update_in, update_out);

   input a;
   input b;
   input update_in;
   output y;
   output update_out;

   wreal a;
   real a_rv;
   wreal b;
   real b_rv;
   wreal y;
   real y_rv;
   reg update_out;

   assign y = y_rv;

   initial
      begin
        assign a_rv = a;
        assign b_rv = b;
      end

   always@update_in
      begin
        $xor2_cpp(a_rv,b_rv,y_rv);
        update_out = update_in;
      end

endmodule

/////////////////////////////////////////////////////

module inv_cpp(a, y, update_in, update_out);

   input a;
   input update_in;
   output y;
   output update_out;

   wreal a;
   real a_rv;
   wreal y;
   real y_rv;
   reg update_out;

   assign y = y_rv;

   initial
      begin
        assign a_rv = a;
      end

   always@update_in
      begin
        $inv_cpp(a_rv,y_rv);
        update_out = update_in;
      end

endmodule

/////////////////////////////////////////////////////

module latch1_cpp(latch, d, q, update_in, update_out);

   input latch;
   input d;
   input update_in;
   output q;
   output update_out;

   wreal latch;
   real latch_rv;
   wreal d;
   real d_rv;
   wreal q;
   real q_rv;
   reg update_out;

   assign q = q_rv;

   initial
      begin
        assign latch_rv = latch;
        assign d_rv = d;
      end

   always@update_in
      begin
        $latch1_cpp(latch_rv,d_rv,q_rv);
        update_out = update_in;
      end

endmodule

/////////////////////////////////////////////////////

module limitamp_cpp(out, in, update_in, update_out);

   input in;
   input update_in;
   output out;
   output update_out;

   wreal in;
   real in_rv;
   wreal out;
   real out_rv;
   reg update_out;

   assign out = out_rv;

   initial
      begin
        assign in_rv = in;
      end

   always@update_in
      begin
        $limitamp_cpp(in_rv,out_rv);
        update_out = update_in;
      end

endmodule

/////////////////////////////////////////////////////

module sub2_cpp(in2, out, in1, update_in, update_out);

   input in2;
   input in1;
   input update_in;
   output out;
   output update_out;

   wreal in2;
   real in2_rv;
   wreal in1;
   real in1_rv;
   wreal out;
   real out_rv;
   reg update_out;

   assign out = out_rv;

   initial
      begin
        assign in2_rv = in2;
        assign in1_rv = in1;
      end

   always@update_in
      begin
        $sub2_cpp(in2_rv,in1_rv,out_rv);
        update_out = update_in;
      end

endmodule

/////////////////////////////////////////////////////

module sampler_cpp(clk, in, out, update_in, update_out);

   input clk;
   input in;
   input update_in;
   output out;
   output update_out;

   wreal clk;
   real clk_rv;
   wreal in;
   real in_rv;
   wreal out;
   real out_rv;
   reg update_out;

   assign out = out_rv;

   initial
      begin
        assign clk_rv = clk;
        assign in_rv = in;
      end

   always@update_in
      begin
        $sampler_cpp(clk_rv,in_rv,out_rv);
        update_out = update_in;
      end

endmodule

/////////////////////////////////////////////////////

module delay_sample_scalar_cpp(out, in, update_in, update_out);

   parameter delay = 0;
   input in;
   input update_in;
   output out;
   output update_out;

   wreal in;
   real in_rv;
   wreal out;
   real out_rv;
   reg update_out;

   assign out = out_rv;

   initial
      begin
        assign in_rv = in;
      end

   always@update_in
      begin
        $delay_sample_scalar_cpp(in_rv,out_rv,delay);
        update_out = update_in;
      end

endmodule

/////////////////////////////////////////////////////

module dff2(qb, clk, d, q, update_in, update_out);

   input clk;
   input d;
   input update_in;
   output qb;
   output q;
   output update_out;
   wreal net26;
   wreal net019;
   wreal clk;
   wreal d;
   wreal qb;
   wreal q;
   wire update_xi19;
   wire update_xi14;
   wire update_xi20;

   latch1_cpp xi19(clk, d, net019, update_in, update_xi19);
   inv_cpp xi14(clk, net26, update_xi19, update_xi14);
   latch1_cpp xi20(net26, net019, q, update_xi14, update_xi20);
   inv_cpp xi17(q, qb, update_xi20, update_out);

endmodule

/////////////////////////////////////////////////////

module rx_samp(threshold, rx_samp_out, rx_clk_in, rx_samp_in, update_in, update_out);

   input threshold;
   input rx_clk_in;
   input rx_samp_in;
   input update_in;
   output rx_samp_out;
   output update_out;
   wreal samp_out;
   wreal net17;
   wreal threshold;
   wreal rx_clk_in;
   wreal rx_samp_in;
   wreal rx_samp_out;
   wire update_xi0;
   wire update_xi1;

   sampler_cpp xi0(rx_clk_in, rx_samp_in, net17, update_in, update_xi0);
   sub2_cpp xi1(threshold, samp_out, net17, update_xi0, update_xi1);
   limitamp_cpp xi2(rx_samp_out, samp_out, update_xi1, update_out);

endmodule

/////////////////////////////////////////////////////

module cdr_logic(cdrdata, early_late, valid_transition, clk_data, cdredge, update_in, update_out);

   input cdrdata;
   input clk_data;
   input cdredge;
   input update_in;
   output early_late;
   output valid_transition;
   output update_out;
   wreal net48;
   wreal net31;
   wreal net52;
   wreal net34;
   wreal net35;
   wreal net27;
   wreal net28;
   wreal net26;
   wreal net51;
   wreal cdrdata;
   wreal clk_data;
   wreal cdredge;
   wreal early_late;
   wreal valid_transition;
   wire update_xi0;
   wire update_xi1;
   wire update_xi4;
   wire update_xi6;
   wire update_xi2;
   wire update_xi5;

   dff2 xi0(net51, clk_data, cdredge, net52, update_in, update_xi0);
   dff2 xi1(net26, clk_data, cdrdata, net48, update_xi0, update_xi1);
   inv_cpp xi4(clk_data, net35, update_xi1, update_xi4);
   xor2_cpp xi6(net31, cdrdata, net48, update_xi4, update_xi6);
   dff2 xi2(net28, net35, net31, valid_transition, update_xi6, update_xi2);
   xor2_cpp xi5(net34, cdrdata, net52, update_xi2, update_xi5);
   dff2 xi3(net27, net35, net34, early_late, update_xi5, update_out);

endmodule

/////////////////////////////////////////////////////

module link_tx_precoderV(tx_data_out, tx_out, update_in, update_out);

   parameter symbol_freq = 0.0000000000e+00;
   parameter trigger_freq = 0.0000000000e+00;
   parameter eq_tap_length = 0.0000000000e+00;
   parameter Ts = 0.0000000000e+00;
   input update_in;
   output tx_data_out;
   output tx_out;
   output update_out;
   wreal tx_clk;
   wreal net18;
   wreal net010;
   wire [31:0] net10;
   wire [31:0] tx_clk_xi35;
   wreal tx_data_out_xi29;
   wreal tx_out;
   wire update_xi0;
   wire update_xi23;
   wire update_xi35_tx_clk;
   wire update_xi29_tx_data_out;
   wire update_xi30;

   constant_cpp #(0) xi0(net18, update_in, update_xi0);
   signal_source_cpp #(`symbol_freq,3) xi23(net18, tx_clk, net010, update_xi0, update_xi23);
   vpp_internal_double_interp_to_int_convert xi35_tx_clk(tx_clk, tx_clk_xi35, update_xi23, update_xi35_tx_clk);
   prbsV xi35(tx_clk_xi35[0:0], tx_data_out);
   vpp_internal_int_to_double_convert xi29_tx_data_out({31'd0,tx_data_out}, tx_data_out_xi29, update_xi35_tx_clk, update_xi29_tx_data_out);
   eq_setting_from_file_cpp #(`eq_tap_length) xi30(net10, update_xi29_tx_data_out, update_xi30);
   tx_eqV_cpp xi29(tx_out, net10, tx_clk, tx_data_out_xi29, update_xi30, update_out);

endmodule

/////////////////////////////////////////////////////

module cdr(clk_edge, clk_data, clk_adapt, rx_edge, rx_data, update_in, update_out);

   parameter clk_freq = 0.0000000000e+00;
   parameter num_maj_votes = 0.0000000000e+00;
   parameter num_cdr_phases = 0.0000000000e+00;
   input rx_edge;
   input rx_data;
   input update_in;
   output clk_edge;
   output clk_data;
   output clk_adapt;
   output update_out;
   wreal net27;
   wreal net26;
   wreal clkref;
   wreal net34;
   wreal net49;
   wire [31:0] phase;
   wreal net46;
   wreal net47;
   wreal net52;
   wreal net53;
   wreal rx_edge;
   wreal rx_data;
   wreal clk_edge;
   wreal clk_data;
   wreal clk_adapt;
   wire update_xi10;
   wire update_xi13;
   wire update_xi12;
   wire update_xi11;
   wire update_xi6;
   wire update_xi8;
   wire update_xi1;

   constant_cpp #(2.7) xi10(net27, update_in, update_xi10);
   signal_source_cpp #(clk_freq,0) xi13(net27, clkref, net26, update_xi10, update_xi13);
   constant_cpp #(`num_maj_votes) xi12(net34, update_xi13, update_xi12);
   divider_ideal_cpp xi11(net34, clk_data, net49, update_xi12, update_xi11);
   cdr_counter_cpp #(`num_cdr_phases) xi6(net49, net47, net46, phase, update_xi11, update_xi6);
   phase_interpolator_cpp #(`num_cdr_phases,clk_freq) xi8(clk_edge, clk_data, clk_adapt, clkref, phase, update_xi6, update_xi8);
   cdr_logic xi1(rx_data, net53, net52, clk_data, rx_edge, update_xi8, update_xi1);
   cdr_maj_vote_cpp #(`num_maj_votes) xi5(net47, net46, net49, clk_data, net52, net53, update_xi1, update_out);

endmodule

/////////////////////////////////////////////////////

module link_rx(threshold_adapt, rx_in, rx_edge_out, rx_data_out, rx_clk, rx_adapt_out, update_in, update_out);

   parameter num_cdr_phases = 0.0000000000e+00;
   parameter num_maj_votes = 0.0000000000e+00;
   parameter symbol_freq = 0.0000000000e+00;
   input threshold_adapt;
   input rx_in;
   input update_in;
   output rx_edge_out;
   output rx_data_out;
   output rx_clk;
   output rx_adapt_out;
   output update_out;
   wreal a38;
   wreal a46;
   wreal a40;
   wreal a44;
   wreal threshold_adapt;
   wreal rx_in;
   wreal rx_edge_out;
   wreal rx_data_out;
   wreal rx_clk;
   wreal rx_adapt_out;
   wire update_xi6;
   wire update_xi2;
   wire update_xi5;
   wire update_xi8;
   wire update_xi1;

   constant_cpp #(0) xi6(a40, update_in, update_xi6);
   rx_samp xi2(a40, rx_edge_out, a38, rx_in, update_xi6, update_xi2);
   constant_cpp #(0) xi5(a44, update_xi2, update_xi5);
   cdr #(`symbol_freq,`num_maj_votes,`num_cdr_phases) xi8(a38, rx_clk, a46, rx_edge_out, rx_data_out, update_xi5, update_xi8);
   rx_samp xi1(a44, rx_data_out, rx_clk, rx_in, update_xi8, update_xi1);
   rx_samp xi0(threshold_adapt, rx_adapt_out, a46, rx_in, update_xi1, update_out);

endmodule

/////////////////////////////////////////////////////

module link_topV(update_in, update_out);

   input update_in;
   output update_out;
   wreal tx_out;
   wire tx_data_out;
   wreal tx_out_delay;
   wreal ch_out;
   wreal rx_clk;
   wreal net7;
   wreal rx_data_out;
   wreal net9;
   wreal net10;
   wire update_xi30;
   wire update_xi12;
   wire update_xi23;
   wire update_xi1;
   wire update_out;

   constant_cpp #(0) xi30(net10, update_in, update_xi30);
   link_tx_precoderV #(`symbol_freq,`trigger_freq,`eq_tap_length,`ts) xi12(tx_data_out, tx_out, update_xi30, update_xi12);
   delay_sample_scalar_cpp #(`delay) xi23(tx_out_delay, tx_out, update_xi12, update_xi23);
   link_channel_cpp xi1(ch_out, tx_out_delay, update_xi23, update_xi1);
   link_rx #(`num_cdr_phases,`num_maj_votes,`symbol_freq) xi2(net10, ch_out, net7, rx_data_out, rx_clk, net9, update_xi1, update_out);

endmodule

/////////////////////////////////////////////////////

/////////  End of Auto-generated Verilog Modules    /////////
