%%% Test bench for High-Speed Link Transceiver adaptation Example %%%
clear all; close all; clc;
set(0,  'DefaultAxesFontSize', 15,...
    'DefaultTextfontSize', 15,...
    'DefaultLineLineWidth', 2);

% Load HspiceToolBox
addpath('/home/vault_stojanovic/software/VppSimShared/HspiceToolbox');  % Load HspiceToolBox

Tsym = 1/8e9;
ratio = 20;
Ts=Tsym/ratio;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Part1: Read and plot the s4p file
if 1
    channelName='channel_data.s4p';
    mode='s21';
    [f,H]=extract_mode_from_s4p(channelName,mode);
    figure(1)
    subplot(211),plot(f*1e-9,20*log10(abs(H)),'b'); xlabel('freqyency [GHz]'); ylabel('Transfer function [dB]'); grid on;
    ylim([-100, 5]);
    title('Channel Transfer Function');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Part2: Generate and plot the channel impulse reponse

    imp=xfr_fn_to_imp(f,H,Ts,Tsym); %%% Ts sampled impulse response
    nsym_short=300; %%% persistance of the impulse response tail in the channel in terms of the number of symbols
    imp_short=imp(1:min(length(imp), floor(nsym_short*Tsym/Ts)));
    pr = conv(imp_short, ones(1, floor(Tsym/Ts)));
    subplot(212), plot((1: length(pr))*Ts, pr,'b.-'); hold on; grid on;
    ylim([-0.2, 0.5]);
    title('Pulse Response');
    save link_channel.dat imp_short -ascii;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Part3: Compute initial ZFE equalizer solution

    eq_tap_length=5;    % length of the pre-emphasis filter
    eq_tap_pre=2;       % length of taps for post-cursor
    eq_taps=zfe(imp_short,Tsym,Ts,eq_tap_length,eq_tap_pre)
    
%    eq_taps(1) = .1;
    eq_taps(2) = .1;
    eq_taps(3) = .1;
%    eq_taps(4) = .1;
%    eq_taps(5) = .1;    
    % eq_taps=[1 0 0 0 0];
    save eq_taps.dat eq_taps -ascii;
end

close all  % comment it if you want to check the previous plot.

%%% Run the simulation
%   cppsim test.par;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Part3.5: Random test
%%% Load the signals after simulation
x=loadsig('test.tr0');
lssig(x)
figure
plotsig(x, ' vec_out_1; vec_out_2'); %% plot tap1 and tap2. It counts from zero

figure
y=loadsig('err.tr0');
lssig(y)
plotsig(y, 'sig, dlevadapt');

% figure
% plotsig(x, 'tx_data_out, tx_out, ch_out');
% ylim([-1.25, 1.25]);

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %% Part4: Eye diagram drawing
% %%% Load the signals after simulation
% 
if 1
    % eye diagram before convergence
    figure
    eyesig(x, 2*Tsym, 0.1e-6,'ch_out, rx_clk');
    
    % eye diagram after convergence
    figure
    eyesig(x, 2*Tsym, 1.5e-6,'ch_out, rx_clk');
end




