%%% Test bench for High-Speed Link Transceiver Example %%%
clear all; close all; clc;
% path = '/home/vault_stojanovic/sanquan/public';
set(0,  'DefaultAxesFontSize', 15,...
    'DefaultTextfontSize', 15,...
    'DefaultLineLineWidth', 2);
addpath('/home/vault_stojanovic/software/VppSimShared/HspiceToolbox');  % Load HspiceToolBox
% addpath('/mit/6.22s/tools/vppsim/VppSimShared/HspiceToolbox');  % Load HspiceToolBox

%% Run it once.
% mex -setup;   % select '2' gcc

%% run it if you change the function link_top
% compile_link_top;


Tsym = 1/8e9;
ratio = 20;
Ts=Tsym/ratio;
eq_tap_length = 5;
num_cdr_phases = ratio;
num_maj_votes = 1024;
delay = 0;
num_sim_steps = 5e5;
symbol_freq = 1/Tsym;
trigger_freq = symbol_freq;

if 1
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Part1: Read and plot the s4p file
    channelName='channel_data.s4p';
    mode='s21';
    [f,H]=extract_mode_from_s4p(channelName,mode);
    figure(1)
    subplot(211),plot(f*1e-9,20*log10(abs(H)),'b'); xlabel('freqyency [GHz]'); ylabel('Transfer function [dB]'); grid on;
    ylim([-100, 5]);
    title('Channel Transfer Function');
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Part2: Generate and plot the channel impulse reponse

    imp=xfr_fn_to_imp(f,H,Ts,Tsym); %%% Ts sampled impulse response
    nsym_short=300; %%% persistance of the impulse response tail in the channel in terms of the number of symbols
    imp_short=imp(1:min(length(imp), floor(nsym_short*Tsym/Ts)));
    pr = conv(imp_short, ones(1, floor(Tsym/Ts)));
    subplot(212), plot((1: length(pr))*Ts, pr,'b.-'); hold on; grid on;
    ylim([-0.2, 0.5]);
    title('Pulse Response');
    save link_channel.dat imp_short -ascii;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Part3: Compute initial ZFE equalizer solution
    % eq_tap_length=5;    % length of the pre-emphasis filter
    eq_tap_pre=1;       % length of taps for post-cursor
    eq_taps=zfe(imp_short,Tsym,Ts,eq_tap_length,eq_tap_pre)
    % eq_taps=[0 1 0 0 0];
    save eq_taps.dat eq_taps -ascii;

end

[tx_out,  tx_out_delay, ch_out, rx_data_out, tx_data_out, ber, counter, del, phase] ...
    = link_top(ratio, symbol_freq, trigger_freq, eq_tap_length, num_cdr_phases, num_maj_votes, delay, num_sim_steps, Ts);
link_top('end');  %% reset the mex simulation state

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Part1: Eye diagram drawing
%%% Load the signals after simulation
if 1
    figure(2)
    hold on;
    grid on;
    tmp = ch_out(100*ratio: end);
    len = floor(length(tmp) / ratio /2);
    for i = 1: len
        plot(Ts * (1 : ratio*2), tmp((i-1)*ratio*2 +1: i*ratio*2));
    end
    xlim([Ts Ts * ratio*2]);
    xlabel('Time (s)');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Part2: CDR output phase
if 1
    figure(3)
    tmp = phase(1: ratio*num_maj_votes: end);
    stem(ratio*num_maj_votes*Ts*(1: length(tmp)), tmp);
    xlabel('Time (s)');  ylabel('CDR output phase');
    ylim([0, 1.2*ratio])
    grid on;
end
