%
%   Written by Michael Perrott for viewing CppSim/VppSim output
%   Distribution:  http://www.cppsim.com
%
%   This code is used as part of the CppSimView program
%   (cppsimview.m)
%
%   For those that use this code - please
%   keep my name attached to it (as currently done).  
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU Library General Public
% License along with this library; if not, write to the MxFree
% Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function [] = deci_plot(x,y,plotnum,plot_args,right_ax_offset,right_ax_label)

% x and y can either be:
%    plot vectors or xmin, xmax values
% plotnum can either be:
%   the subplot number (when x and y are plot vectors)
%   a command directive when x=y (scalar values)
%       subplot == -1:  pan left
%       subplot == 1:  pan right
%       subplot == -2:  zoom in
%       subplot == 2:  zoom out
%       subplot == 0:  fit to screen
%       subplot == anything else:  refresh

global xlabel_string title_string legend_string_global ylabel_string
global plot_axes_handle
persistent x1 y1 x2 y2 x3 y3 x4 y4 x5 y5 x6 y6 x7 y7 x8 y8
global num_subplots cur_marker min_x_global plot_linewidth
persistent min_x max_x last_plot_args last_right_ax_offset last_right_ax_label
persistent prev_min_x prev_max_x

global fft_flag
fft_flag = 0;

if length(x) == 0 | length(y) == 0
    return
end

if size(x,2) > size(x,1)
   x = x';
end
if size(y,2) > size(y,1)
   y = y';
end
  
if length(x) > 1
   %%  here x and y are plot vectors

   if plotnum < 1
      'error in deci_plot:  plotnum must be >= 1' 
      return;
   elseif plotnum > num_subplots+1
      'error in deci_plot: plotnum too high'
      return
   elseif plotnum > 8
      'error in deci_plot: plotnum too high (can''t be greater than 8)'
      return
   end
   num_subplots = plotnum;   

   if plotnum == 1
      x1 = x;
      y1 = y;
      clear x2 y2 x3 y3 x4 y4 x5 y5 x6 y6 x7 y7 x8 y8
   elseif plotnum == 2
      x2 = x;
      y2 = y;
      clear x3 y3 x4 y4 x5 y5 x6 y6 x7 y7 x8 y8
   elseif plotnum == 3
      x3 = x;
      y3 = y;
      clear x4 y4 x5 y5 x6 y6 x7 y7 x8 y8
   elseif plotnum == 4
      x4 = x;
      y4 = y;
      clear x5 y5 x6 y6 x7 y7 x8 y8
   elseif plotnum == 5
      x5 = x;
      y5 = y;
      clear x6 y6 x7 y7 x8 y8
   elseif plotnum == 6
      x6 = x;
      y6 = y;
      clear x7 y7 x8 y8
   elseif plotnum == 7
      x7 = x;
      y7 = y;
      clear x8 y8
   elseif plotnum == 8
      x8 = x;
      y8 = y;
   end

   min_x = 1;
   max_x = length(x1);

   if nargin > 3
      if iscell(plot_args) == 1
         last_plot_args = char(plot_args);
      else
         last_plot_args = plot_args;
      end
   else
      last_plot_args = ' ';
   end

   if strfind(last_plot_args,'nodec') > 0
      x_new = x;
      y_new = y;
   else
      [x_new,y_new] = min_max_decimate(x,y);
   end

   if length(x_new) == 1
      x_new = x;
      y_new = y;
   end

   if nargin > 3
      if strfind(last_plot_args,'fftdb') > 0
         fft_flag = 2;
      elseif strfind(last_plot_args,'fft') > 0
         fft_flag = 1;
      end
      if strfind(last_plot_args,'logx') > 0
           semilogx(x_new,y_new);
      elseif strfind(last_plot_args,'logy') > 0
           semilogy(x_new,y_new);
      elseif strfind(last_plot_args,'logxy') > 0 | ...
             strfind(last_plot_args,'loglog') > 0
           loglog(x_new,y_new);
      else
           plot(x_new,y_new);
      end
      if strfind(last_plot_args,'right_ax') > 0
           if nargin > 5
              last_right_ax_offset = right_ax_offset;
              last_right_ax_label = char(right_ax_label);
              cur_ax = gca;
              add_right_axis(cur_ax,last_right_ax_offset,last_right_ax_label);
              axes(cur_ax);
           else
 	      disp('error:  not enough arguments with ''right_ax'' arg');
              return;
           end
      end
   else
      plot(x_new,y_new);
   end

   cur_ax = gca;
   min_t = min(x_new(:,1));
   max_t = max(x_new(:,1));
   if max_t > min_t
      set(cur_ax,'XLim',[min_t max_t]);
   end
   y_ax = get(cur_ax,'YLim');
   y1_ax = y_ax(1);
   y2_ax = y_ax(2);
   delta_y = (y2_ax-y1_ax)*0.1;
   set(cur_ax,'YLim',[y1_ax-delta_y y2_ax+delta_y]);
   grid on;
   cur_marker = 'none';
%% update linewidth
   if (plot_linewidth > 0)
       hli=get(cur_ax,'Children'); % get the handle of lines in axis
       for i=1:length(hli)
          set(hli(i),'LineWidth',plot_linewidth);
       end
   end

else
   %%  here x = xmin, y = xmax

   if length(x1) == 0 | length(y1) == 0
      return
   end

   if x ~= y
       prev_min_x = min_x;
       prev_max_x = max_x;

       ind = find(x1 >= x);
       if length(ind) > 0
          min_x = ind(1);
       else
          min_x = 1;
       end

       ind = find(x1 >= y);
       if length(ind) > 0
          max_x = ind(1);
       else
          max_x = length(x1);
       end

       if prev_min_x == min_x & prev_max_x == max_x
          return;
       end
   else  %% x=y implies that we use previous min_x,max_x
       prev_min_x = min_x;
       prev_max_x = max_x;

       if plotnum == 0 %% back to full scale
          min_x = 1;
          max_x = length(x1);
       elseif plotnum == -1  %% pan left
          delta_x = ceil((max_x-min_x)/10.0);
          if (min_x - delta_x) >= 1
             min_x = min_x - delta_x;
             max_x = max_x - delta_x;
          end
       elseif plotnum == 1  %% pan right
          delta_x = ceil((max_x-min_x)/10.0);
          if (max_x + delta_x) <= length(x1)
             min_x = min_x + delta_x;
             max_x = max_x + delta_x;
          end
       elseif plotnum == -2  %% zoom in
          delta_x = ceil((max_x-min_x)/4.0);
          if delta_x > 10
              min_x = min_x + delta_x;
              max_x = max_x - delta_x;
          end
       elseif plotnum == 2  %% zoom out
          delta_x = ceil((max_x-min_x)/2.0);
          if (min_x - delta_x) >= 1
              min_x = min_x - delta_x;
          else
	      min_x = 1;
          end
          if (max_x + delta_x) <= length(x1)
              max_x = max_x + delta_x;
	  else
 	      max_x = length(x1);
          end
       end
       if prev_min_x == min_x & prev_max_x == max_x & plotnum ~= 3
          return;
       end
       subplot(num_subplots,1,num_subplots);

   end

   for i = 1:num_subplots
      clear x;
      clear y;
      if i == 1
         x = x1(min_x:max_x,:);
         y = y1(min_x:max_x,:);
      elseif i == 2
         x = x2(min_x:max_x,:);
         y = y2(min_x:max_x,:);
      elseif i == 3
         x = x3(min_x:max_x,:);
         y = y3(min_x:max_x,:);
      elseif i == 4
         x = x4(min_x:max_x,:);
         y = y4(min_x:max_x,:);
      elseif i == 5
         x = x5(min_x:max_x,:);
         y = y5(min_x:max_x,:);
      elseif i == 6
         x = x6(min_x:max_x,:);
         y = y6(min_x:max_x,:);
      elseif i == 7
         x = x7(min_x:max_x,:);
         y = y7(min_x:max_x,:);
      elseif i == 8
         x = x8(min_x:max_x,:);
         y = y8(min_x:max_x,:);
      end

      if strfind(last_plot_args,'nodec') > 0
         x_new = x;
         y_new = y;
      else
         [x_new,y_new] = min_max_decimate(x,y);
      end
      if length(x_new) == 1
         x_new = x;
         y_new = y;
      end

      subplot(num_subplots,1,i);

      if strcmp(last_plot_args,' ') == 0
          if strfind(last_plot_args,'fftdb') > 0
             fft_flag = 2;
          elseif strfind(last_plot_args,'fft') > 0
             fft_flag = 1;
          end
          if strfind(last_plot_args,'logx') > 0
              semilogx(x_new,y_new);
          elseif strfind(last_plot_args,'logy') > 0
              semilogy(x_new,y_new);
          elseif strfind(last_plot_args,'logxy') > 0 | ...
                 strfind(last_plot_args,'loglog') > 0
              loglog(x_new,y_new);
          else
              plot(x_new,y_new);
          end
          if strfind(last_plot_args,'right_ax') > 0
              cur_ax = gca;
              add_right_axis(cur_ax,last_right_ax_offset,last_right_ax_label);
              axes(cur_ax);
          end
      else
         plot(x_new,y_new);
      end

      cur_ax = gca;
      min_t = min(x_new(:,1));
      max_t = max(x_new(:,1));
      if max_t > min_t
         set(cur_ax,'XLim',[min_t max_t]);
      end
      y_ax = get(cur_ax,'YLim');
      y1_ax = y_ax(1);
      y2_ax = y_ax(2);
      delta_y = (y2_ax-y1_ax)*0.1;
      set(cur_ax,'YLim',[y1_ax-delta_y y2_ax+delta_y]);
      set(cur_ax,'XGrid','on');
      set(cur_ax,'YGrid','on');
      if length(legend_string_global) >= i
         h_leg = legend(char(legend_string_global{i}));
         set(h_leg,'TextColor',[1 1 1]);
      end

%% update linewidth
      if plot_linewidth > 0 | strcmp(cur_marker,'o') == 1
          hli=get(cur_ax,'Children'); % get the handle of lines in axis
          for i=1:length(hli)
             if plot_linewidth > 0
                set(hli(i),'LineWidth',plot_linewidth);
             end
             if strcmp(cur_marker,'o') == 1
                set(hli(i),'Marker','o');
             end
          end
      end
   end

   create_ylabel;
   create_xlabel;
   create_title;

end
min_x_global = min_x;
plot_axes_handle = cur_ax;



