% Evaluate expression, invoking signal names from structure x.
% Usage: y = evalsig(x, expr)
% Note that numeric node names should be prepended by '@' to
% distinguish them from constants.  If the global variable
% 'sweep' exists and is non-zero, a single curve is selected
% from multiple SPICE parameter sweeps.  Setting sweep=0
% returns data for all sweeps.
%
%   Written by Mike Perrott 
%   Sweep selection added by Scott Willingham
%   Updated with feval for CppSim viewer by Mike Perrott in Mar '04

function [y] = evalsig(x,expr)


c = parse_expression(x,expr);

if length(c) == 0
   y = [];
   return;
end

length_c = length(c);

while 1
   flag = 0;
   for i = 1:length_c
      if strcmp(char(c(i)),'-') == 1 && i < length_c
          if i == 1
             flag = 1;
          elseif strcmp(char(c(i-1)),'(') == 1
	     flag = 1;
          end
          if flag == 1
  	     if strcmp(c(i+1),'(') == 0
                c(i+1) = cellstr(strcat('-',char(c(i+1))));
                for j = i+1:length_c
   	           c(j-1) = c(j);
                end
  	        length_c = length_c - 1;
                break;
             else
                new_c = cell(length_c+1,1);
                new_c(1) = cellstr('0');
                for i = 1:length_c
          	   new_c(i+1) = c(i);
                end
                c = new_c;
                length_c = length(c);
             end
          end
      end
   end
   if flag == 0
       break;
   end
end

while 1
   flag = 0;
   for i = 1:length_c
      if strcmp(char(c(i)),'+') == 1 && i < length_c
          if i == 1
             flag = 1;
          elseif strcmp(char(c(i-1)),'(') == 1
	     flag = 1;
          end
          if flag == 1
  	     if strcmp(c(i+1),'(') == 0
                for j = i+1:length_c
   	           c(j-1) = c(j);
                end
  	        length_c = length_c - 1;
                break;
             else
                new_c = cell(length_c+1,1);
                new_c(1) = cellstr('0');
                for i = 1:length_c
          	   new_c(i+1) = c(i);
                end
                c = new_c;
                length_c = length(c);
             end
          end
      end
   end
   if flag == 0
       break;
   end
end

m = 1;
result(:,1) = zeros(length(x(1).data),1);

while 1
    j = 0;
    for i = 1:length_c
        var = char(c(i));
        if var == '('
           j = i;
        end
    end
    if j == 0
	result(:,m) = eval_no_paren_expr(x,result,c(1:length_c));
        break;
    else
        k = 0;
        for i = j+1:length_c
            var = char(c(i));
            if var == ')'
	       k = i;
               break;
            end 
        end
        if k == 0
           'error in evalsig'
        end
	result(:,m) = eval_no_paren_expr(x,result,c(j+1:k-1));
        c(j) = cellstr(sprintf('z(%d)',m));
        m = m+1;
        for i = k+1:length_c
            j = j+1;
            c(j) = c(i);
        end
	length_c = j;
    end
end
%%'this approach'
%%result(1:4,m)

y = result(:,m);
clear result;



function [oper,sig_num,sign] = eval_sig_num(inp)

oper = -2;
sign = 1;
sig_num = -1;

if strncmp(inp,'x(',2) == 1
    [t,r] = strtok(inp(3:length(inp)),')');
    sig_num = str2num(t); 
elseif strncmp(inp,'-x(',3) == 1
    [t,r] = strtok(inp(4:length(inp)),')');
    sig_num = str2num(t); 
    sign = -1;
elseif strncmp(inp,'z(',2) == 1
    oper = -1;
    [t,r] = strtok(inp(3:length(inp)),')');
    sig_num = str2num(t); 
elseif strncmp(inp,'t(',2) == 1
    oper = 0;
    [t,r] = strtok(inp(3:length(inp)),')');
    sig_num = str2num(t); 
else
    [t,r] = strtok(inp,'+-*/.^');
    if length(t) == 0
       oper = 1;
    else
       [t,r] = strtok(inp,'0123456789.e-+');
       if length(t) > 0
          if t(1) == ')'
             oper = 3;
          elseif t(1) == '('
             oper = 4;
          else
             oper = 2;
          end
       end
    end
end

function result = eval_no_paren_expr(x,prev_result,expr_in)

m = 1;
result_temp(:,m) = zeros(length(x(1).data),1);

expr = expr_in;
length_expr = length(expr);

safeguard_counter = 1;
while 1
   if safeguard_counter > 1000
      break;
   else
      safeguard_counter = safeguard_counter + 1;
   end

   flag = 0;
   for i=1:length_expr
      func = char(expr(i));
      [oper,sig_num,sign] = eval_sig_num(func);

      if oper == 2  %% function
          flag = 1;
          if i < length_expr
             var = char(expr(i+1));
             val = eval_var(var,x,prev_result,result_temp);

             if func(1) == '-'
  	        func = func(2:length(func));
                result_temp(:,m) = -feval(func,val);
             else
                result_temp(:,m) = feval(func,val);
             end
             expr(i) = cellstr(sprintf('t(%d)',m));
             for k = i+2:length_expr
                expr(k-1) = expr(k);
             end
             length_expr = length_expr-1; 
             m = m+1;
          else
             'error in eval_no_paren_expr'
          end
	  break;
      end
   end
   if flag == 0
      break;
   end
end

safeguard_counter = 1;
while 1
   if safeguard_counter > 1000
      break;
   else
      safeguard_counter = safeguard_counter + 1;
   end

   flag = 0;
   for i=1:length_expr
      func = char(expr(i));

      if strcmp(func,'.^') == 1 
          flag = 1;
          if i < length_expr & i > 1
             var = char(expr(i+1));
             val = eval_var(var,x,prev_result,result_temp);
             var = char(expr(i-1));
             val2 = eval_var(var,x,prev_result,result_temp);

             result_temp(:,m) = feval('power',val2,val);
             expr(i-1) = cellstr(sprintf('t(%d)',m));
             for k = i+2:length_expr
                expr(k-2) = expr(k);
             end
             length_expr = length_expr-2; 
             m = m+1;
          else
             'error in eval_no_paren_expr'
          end
	  break;
      end
   end
   if flag == 0
      break;
   end
end

safeguard_counter = 1;
while 1
   if safeguard_counter > 1000
      break;
   else
      safeguard_counter = safeguard_counter + 1;
   end

   flag = 0;
   for i=1:length_expr
      func = char(expr(i));

      if strcmp(func,'.*') == 1  | strcmp(func,'./') == 1
          flag = 1;
          if i < length_expr & i > 1
             var = char(expr(i+1));
             val = eval_var(var,x,prev_result,result_temp);
             var = char(expr(i-1));
             val2 = eval_var(var,x,prev_result,result_temp);

             if strcmp(func,'.*') == 1
                result_temp(:,m) = feval('mult',val2,val);
             else
                result_temp(:,m) = feval('div',val2,val);
             end
             expr(i-1) = cellstr(sprintf('t(%d)',m));
             for k = i+2:length_expr
                expr(k-2) = expr(k);
             end
             length_expr = length_expr-2; 
             m = m+1;
          else
             'error in eval_no_paren_expr'
          end
	  break;
      end
   end
   if flag == 0
      break;
   end
end

safeguard_counter = 1;
while 1
   if safeguard_counter > 1000
      break;
   else
      safeguard_counter = safeguard_counter + 1;
   end

   flag = 0;
   for i=1:length_expr
      func = char(expr(i));

      if strcmp(func,'+') == 1  | strcmp(func,'-') == 1
          flag = 1;
          if i < length_expr & i > 1
             var = char(expr(i+1));
             val = eval_var(var,x,prev_result,result_temp);
             var = char(expr(i-1));
             val2 = eval_var(var,x,prev_result,result_temp);

             if strcmp(func,'+') == 1
                result_temp(:,m) = feval('plus',val2,val);
             else
                result_temp(:,m) = feval('minus',val2,val);
             end
             expr(i-1) = cellstr(sprintf('t(%d)',m));
             for k = i+2:length_expr
                expr(k-2) = expr(k);
             end
             length_expr = length_expr-2; 
             m = m+1;
          else
             'error in eval_no_paren_expr'
          end
	  break;
      end
   end
   if flag == 0
      break;
   end
end

if m == 1
   var = char(expr(1));
   result = eval_var(var,x,prev_result,result_temp);
else
   result = result_temp(:,m-1);
end
clear result_temp;



function val = eval_var(var,x,prev_result,result_temp)

[oper,sig_num,sign] = eval_sig_num(var);
if oper == -2
    if sig_num > 0
        val = sign*x(sig_num).data;
    else
        val = str2num(var);
    end
elseif oper == -1
    val = prev_result(:,sig_num);
elseif oper == 0
    val = result_temp(:,sig_num);
else
    var
    'error in eval_var'
end
