#include "mex.h"

void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
double *x_new,*y_new,*x,*y;
int rows_x, cols_x, rows_y, cols_y, step, half_step;
int i,j,k,num_plot_points, final_j_x, final_j_y, offset;
double x_first, min_val, max_val, step_real, half_step_real, offset_real;
int j2, final_j_x2, final_j_y2, offset_next;

if (nrhs != 2)
    mexErrMsgTxt("Error in usage - should be:  [x_new,y_new] = min_max_decimate([x,y])");
if (!mxIsDouble(prhs[0]))
    mexErrMsgTxt("Error: x must be double valued matrix");
if (!mxIsDouble(prhs[1]))
    mexErrMsgTxt("Error: y must be double valued matrix");
if (nlhs != 2)
    mexErrMsgTxt("Error: missing output variable(s).  Usage:  [x_new,y_new] = min_max_decimate([x,y])");

if (mxGetNumberOfDimensions(prhs[0]) != 2)
    mexErrMsgTxt("Error: number of dimensions of x must be 2!\n");
if (mxGetNumberOfDimensions(prhs[1]) != 2)
    mexErrMsgTxt("Error: number of dimensions of y must be 2!\n");


num_plot_points = 4000;

rows_x = (mxGetDimensions(prhs[0]))[0];
cols_x = (mxGetDimensions(prhs[0]))[1];
rows_y = (mxGetDimensions(prhs[1]))[0];
cols_y = (mxGetDimensions(prhs[1]))[1];

if (rows_x != rows_y)
    mexErrMsgTxt("Error: number of dimensions of x and y must be the same!\n");

x = mxGetPr(prhs[0]);
y = mxGetPr(prhs[1]);

x_first = x[0];

/* don't decimate unless we can achieve at least a factor 
   of 2 reduction in the number of points */
if (rows_x <= 2*num_plot_points)
  {
   /* create MATLAB output matrices of length 1 */
   plhs[0] = mxCreateDoubleMatrix(1,1,mxREAL);
   plhs[1] = mxCreateDoubleMatrix(1,1,mxREAL);
   x_new = mxGetPr(plhs[0]);
   y_new = mxGetPr(plhs[1]);
   x_new[0] = -1.0;
   y_new[0] = -1.0;
   return;
  }

/*  don't decimate eye diagrams */
for (i = 1; i < rows_x; i++)
   if (x[i] == x_first)
      {
      /* create MATLAB output matrices of length 1 */
      plhs[0] = mxCreateDoubleMatrix(1,1,mxREAL);
      plhs[1] = mxCreateDoubleMatrix(1,1,mxREAL);
      x_new = mxGetPr(plhs[0]);
      y_new = mxGetPr(plhs[1]);
      x_new[0] = -1.0;
      y_new[0] = -1.0;
      return;
      }


half_step_real = ((double) rows_x)/((double) num_plot_points);
step_real = half_step_real*2.0;

half_step = (int) half_step_real;
step = (int) step_real;

/*
mexPrintf("step = %5.3e, half_step = %5.3e\n",step_real,half_step_real); 
mexPrintf("step = %d, half_step = %d\n",step,half_step); 
*/

/* create MATLAB output matrices */
plhs[0] = mxCreateDoubleMatrix(num_plot_points,cols_x,mxREAL);
plhs[1] = mxCreateDoubleMatrix(num_plot_points,cols_y,mxREAL);

x_new = mxGetPr(plhs[0]);
y_new = mxGetPr(plhs[1]);

final_j_x = cols_x*num_plot_points;
final_j_y = cols_y*num_plot_points;
final_j_x2 = cols_x*rows_x;
final_j_y2 = cols_y*rows_y;

offset_real = 0.0;
for (i = 0; i < num_plot_points; i += 2)   
   {
    offset = (int) offset_real;
    offset_next = (int) (offset_real + step_real);
  
    for (j = 0, j2 = 0; j < final_j_x & j2 < final_j_x2; 
                        j += num_plot_points, j2 += rows_x)
       {
	x_new[i + j] = x[offset + j2];
	x_new[i + j + 1] = x[offset + half_step + j2];
       }   
    for (j = 0, j2 = 0; j < final_j_y & j2 < final_j_y2; 
                        j += num_plot_points, j2 += rows_y)
       {
	min_val = y[offset + j2];
	max_val = y[offset + j2];
	for (k = 0; (offset + k) < offset_next && (offset + k) < rows_y; k++)
	   {
	    if (y[offset + k + j2] < min_val)
	       min_val = y[offset + k + j2];
	    if (y[offset + k + j2] > max_val)
	       max_val = y[offset + k + j2];
	   }
	y_new[i + j] = min_val;
	y_new[i + j + 1] = max_val;
       }   
    offset_real += step_real;
   }

return;
}



