%
%   Written by Michael Perrott for viewing CppSim/VppSim output
%   Distribution:  http://www.cppsim.com
%
%   This code is used as part of the CppSimView program
%   (cppsimview.m)
%
%   For those that use this code - please
%   keep my name attached to it (as currently done).  
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU Library General Public
% License along with this library; if not, write to the MxFree
% Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

% Parse expression, invoking signal names from structure x.
% Usage: y = parse_expression(x, expr)
%
%   Written for CppSim viewer by Michael Perrott in Mar '04

function c = parse_expression(x,expr)

global sweep; % selects just one curve of data set

expr = lower(expr);
saved_expr = expr;

%% replace ^ with .^, * with .*, / with ./ 
%% unless they are already preceeded by .
expr = strrep(expr,'.^','^');
expr = strrep(expr,'.*','*');
expr = strrep(expr,'./','/');

expr = strrep(expr,'^','&^');
expr = strrep(expr,'*','&*');
expr = strrep(expr,'/','&/');

% expr = strrep(expr,'pi','3.14159265358979');

e_indices = strfind(expr,'e');
for i = 1:length(e_indices)
    index = e_indices(i);
    if index > 1 & index < length(expr)
       e_expr = expr(index-1:index);
       [t,r] = strtok(e_expr,'0123456789.');
       if t(1) == 'e'
          if expr(index+1) == '-'
             expr(index+1) = '$';
          elseif expr(index+1) == '+'
             expr(index+1) = '%';
          end
       end
    end
end


%% parse expression

delim = '+-/*^()&:, ';
delim_m = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_0123456789.@';


for j = 1:length(expr)
   if (expr(j) ~= ' ' & expr(j) ~= '-' & expr(j) ~= '(' & expr(j) ~= '+')
      break;
   end
end

i = 1;
expr_in = expr;

if (j > 1)
   while 1
      for k = 1:length(expr_in)
         if expr_in(k) ~= ' '
            break;
         end
      end
      if k == j
         break;
      end
      if expr_in(k) == '('
         c(i) = cellstr('(');
         i = i + 1;
         expr_in(k) = ' ';
      elseif expr_in(k) == '-'
         c(i) = cellstr('-');
         i = i + 1;
         expr_in(k) = ' ';
      elseif expr_in(k) == '+'
         c(i) = cellstr('+');
         i = i + 1;
         expr_in(k) = ' ';
      end
   end
end


r = expr(j:length(expr));


while(1)
   if length(r) > 0
      [t,r] = strtok(r,delim);
      if length(t) > 0
         c(i) = cellstr(t);
         i = i+1;
      end
      [t,r] = strtok(r,delim_m);
      flag = 0;
      for k = 1:length(t)
         if flag == 1
             flag = 0;
         elseif t(k) ~= ' ' & t(k) ~= '&'
            c(i) = cellstr(t(k));
	    i = i+1;
         elseif t(k) == '&'
            c(i) = cellstr(t(k:k+1));
            i = i+1;      
            flag = 1;
         end
      end
   else
      break;
   end
end


%% prepend all node names that are pure numbers with @
for i=1:length(x)
  temp_string = x(i).name;
  if sum(isletter(temp_string)) == 0
    x(i).name = strcat('@',x(i).name);
  end
end

%% substitute structure names for spice signals names
for i = 1:length(x)
   match_vec = strmatch(lower(x(i).name),c,'exact');

   if length(match_vec) > 0
      for k=1:length(match_vec)
         if match_vec(k) < length(c)
            if strncmp(c(match_vec(k)+1),'(',1) ~= 1 %%avoid subst of functions
               szdata = size(x(i).data);
               if sweep > 0 & sweep <= szdata(2)
		   c(match_vec(k))=cellstr(sprintf('x(%d).data(:,%d)',i,sweep));

               else
		   c(match_vec(k))=cellstr(sprintf('x(%d).data',i));
               end
            end
         else
               szdata = size(x(i).data);
               if sweep > 0 & sweep <= szdata(2)
		   c(match_vec(k))=cellstr(sprintf('x(%d).data(:,%d)',i,sweep));

               else
		   c(match_vec(k))=cellstr(sprintf('x(%d).data',i));
               end
         end
      end
   end
end

%% substitute pi with value
for i = 1:length(x)
   match_vec = strmatch('pi',c,'exact');

   if length(match_vec) > 0
      for k=1:length(match_vec)
         if match_vec(k) < length(c)
            if strncmp(c(match_vec(k)+1),'(',1) ~= 1 %%avoid subst of functions
               szdata = size(x(i).data);
               if sweep > 0 & sweep <= szdata(2)
		   c(match_vec(k))=cellstr('3.14159265358979');

               else
		   c(match_vec(k))=cellstr('3.14159265358979');
               end
            end
         else
               szdata = size(x(i).data);
               if sweep > 0 & sweep <= szdata(2)
		   c(match_vec(k))=cellstr('3.14159265358979');

               else
		   c(match_vec(k))=cellstr('3.14159265358979');
               end
         end
      end
   end
end

for i = 1:length(c)
   c(i) = cellstr(strrep(char(c(i)),'&','.'));
   c(i) = cellstr(strrep(char(c(i)),'$','-'));
   c(i) = cellstr(strrep(char(c(i)),'%','+'));
end

paren_count = 0;
for i = 1:length(c)
   if strcmp(char(c(i)),'(') == 1
      paren_count = paren_count + 1;
   elseif strcmp(char(c(i)),')') == 1
      paren_count = paren_count - 1;
   end
end
if paren_count ~= 0
   err_message = sprintf('Error: parenthesis mismatch in expression ''%s''',saved_expr);     

   error_box(err_message,'Error in Plot Expression');
   c = [];
   return
end

for i = 1:length(c)
   [t,r] = strtok(char(c(i)),'+-*/^.0123456789)');
   if length(t) > 0
      if t(1) == 'e'
         [t,r] = strtok(t(2:length(t)),'+-0123456789)');
      end
   end
   if length(t) > 0
      if strncmp(t,'x(',2) == 0 & strcmp(t,'(') == 0
          if i == length(c)
             err_message = sprintf('Error: variable ''%s'' is not defined in expression ''%s''',char(c(i)),saved_expr);     

             error_box(err_message,'Error in Plot Expression');
             c = [];
             return;
          elseif strcmp(char(c(i+1)),'(') == 0
             err_message = sprintf('Error: variable ''%s'' is not defined in expression ''%s''',char(c(i)),saved_expr);     

             error_box(err_message,'Error in Plot Expression');
             c = [];
             return;
          elseif strcmp(char(c(i+1)),'(') == 1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% add all supported functions here %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
             switch lower(char(c(i)))
                case 'fft'
                case 'log'
                case 'log10'
                case 'sin'
                case 'cos'
                case 'exp'
                case 'abs'
                case 'exp'
                case 'tan'
                case 'atan'
                case 'mean'
                case 'var'
                case 'std'
                case 'sqrt'
                case 'real'
                case 'imag'
                case 'conj'
                case 'angle'
                case 'sign'
                case 'round'
                case 'floor'
                case 'ceil'
                case 'complex'
                case 'unwrap'
                case 'db'
                case 'ph'
                otherwise
                   err_message = sprintf('Error: function ''%s'' is not supported at this time in expression ''%s'' --- add function to parse\_expression.m and check\_arg.m to correct this issue',char(c(i)),saved_expr);     
                   error_box(err_message,'Error in Plot Expression');
                   c = [];
                   return
             end
          end
      end
   end
end
