%
%   Written by Michael Perrott for viewing CppSim/VppSim output
%   Distribution:  http://www.cppsim.com
%
%   This code is used as part of the CppSimView program
%   (cppsimview.m)
%
%   For those that use this code - please
%   keep my name attached to it (as currently done).  
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU Library General Public
% License along with this library; if not, write to the MxFree
% Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

% FFT Plot expressions invoking signal names from structure x.
% Usage: plot_fft_db_logx(x, 'nodes')
%  written by Mike Perrott

function status = plot_fft_db_logx(x,nodes)

global legend_string_global;

%%%%%%%%%%%%%%%%%%%%% Error checking %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin < 2 | nargin > 2
     mssg = sprintf('Error:  must have arguments: plot_fft_db_logx(x,''nodes'')');
     error_box(mssg,'Error using ''plot_fft_db_logx''');
     return 
end
%%%%%%%%%%%%%%%%%%%%% Labels  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

x_labels = {'Freq (GHz)' 'Freq (MHz)' 'Freq (kHz)' 'Freq (Hz)'};
x_limits = [1e9 1e6 1e3 1];

ylabel_name = ' ';
if sum(nodes == ';') > 0 | sum(nodes == ',') > 0
   title_name = 'FFT of CppSim Simulated Signals';
else
   title_name = 'FFT of CppSim Simulated Signal';
end


clf
status = -1;

%%%%%%%%%%%%%%%%%%%%% x definition %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
time = x(1).data;
if length(time) < 20
    return
end
num_samples = length(time);
num_samples_exp = ceil(log2(num_samples));
num_samples = 2^num_samples_exp;
f_length = ceil(num_samples/2)+1;
Ts_vec = filter([1 -1],1,time);
if length(Ts_vec) > 10010
   Ts = mean(Ts_vec(10:10010));
else
   Ts = mean(Ts_vec(10:length(Ts_vec)));
end

xsig = 1/(Ts*num_samples)*(0:(f_length-1));

[xscaled,xlabel_name] = scale_and_label_x(xsig,x_limits,x_labels);

%%%%%%%%%%%%%%%%%%%%% Loop through subplots   %%%%%%%%%%%%%%%%%%%%%%
nsp = 2*(1 + sum(nodes == ';'));	% number of subplots
subplot(nsp,1,1);

for i = 1:2:nsp
    subplot(nsp,1,i);
%%%%%%%%%%%%%%%%%%%%% y definition %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    [ysig,legend_string] = create_ysignals_and_legend(x,nodes,i);
    if length(ysig) == 0
       return
    end
    ysig = fft(ysig,num_samples);
    ysig = ysig(1:f_length,:);
%%%%%%%%%%%%%%%%%%%%% Plot function %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    deci_plot(xscaled,db(ysig),i,'logx,fftdb,nodec');
    h_leg = legend(legend_string);
    set(h_leg,'TextColor',[1 1 1]);
    subplot(nsp,1,i+1);
    deci_plot(xscaled,ph(ysig),i+1,'logx,fftdb,nodec');

%%%%%%%%%%%%%%%%%%%%% Labeling %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    legend_string_global{i+1} = legend_string;
    h_leg = legend(legend_string);
    set(h_leg,'TextColor',[1 1 1]);
end

create_title(title_name);
create_ylabel(ylabel_name);
create_xlabel(xlabel_name); 

status = 0;
