%
%   Written by Michael Perrott for viewing CppSim/VppSim output
%   Distribution:  http://www.cppsim.com
%
%   This code is used as part of the CppSimView program
%   (cppsimview.m)
%
%   For those that use this code - please
%   keep my name attached to it (as currently done).  
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU Library General Public
% License along with this library; if not, write to the MxFree
% Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

%%% This Matlab code is used to plot jitter of a CDR
%  written by Michael Perrott for CppSimView in March 2004
function status = plot_pll_jitter_vs_time(x,ref_timing_node,start_time_in,nodes)

global legend_string_global   %%% need to add rms jitter values to legend
%%%%%%%%%%%%%%%%%%%%% Error checking %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
status = -1;

 if nargin ~= 4
     mssg = sprintf('Error:  must have 4 arguments: plot_pll_jitter_vs_time(x,''ref_timing_node'',start_time,''nodes'')');
     error_box(mssg,'Error using ''plot_pll_jitter_vs_time''');
     return 
 end

 time = evalsig(x,'TIME');

%%%%%%%%%%%%%%%%%%%%%% error checking %%%%%%%%%%%%%%%%%%%%%%%%%%%
 if length(time) < 2
     mssg = sprintf('Error:  must have at least 2 time samples');
     error_box(mssg,'Error using ''plot_pll_jitter_vs_time''');
     return;
 end

 if ischar(start_time_in) == 1
    start_time = str2num(start_time_in);
 else
    start_time = start_time_in;
 end
 if start_time <= 0
    start_time = 0.0;
 end


 nsp = 1 + sum(ref_timing_node == ';');	% number of subplots
 if nsp > 1
     mssg = sprintf('Error:  can''t have '';'' characters in ref_timing_node string --- expr = ''%s''',ref_timing_node);
     error_box(mssg,'Error using ''plot_pll_jitter_vs_time''');
     return 
 end

 nsigs = 1 + sum(ref_timing_node == ',');	% number of signals
 if nsigs > 1
     mssg = sprintf('Error:  can''t have '','' characters in ref_timing_node string --- expr = ''%s''',ref_timing_node);
     error_box(mssg,'Error using ''plot_pll_jitter_vs_time''');
     return 
 end

%%%%%%%%%%%%%%%%%%%%% Labels  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

x_labels = {'Time (seconds)' 'Time (milliseconds)' 'Time (microseconds)' ...
            'Time (nanoseconds)' 'Time (picoseconds)'};
x_limits = [1 1e-3 1e-6 1e-9 1e-12];

jitter_labels = {'s' 'ms' 'microsec' 'ns' 'ps' 'fs'};
jitter_limits = [1 1e-3 1e-6 1e-9 1e-12 1e-15];

ylabel_name = 'Instantaneous Phase (U.I.)';
title_name = 'CppSim Simulated PLL Phase';

clf

%%%%%%%%%%%%%%%%%%%%% x definition (first part) %%%%%%%%%%%%%%%%%%%%%%%%%%%%

refsig = evalsig(x,ref_timing_node);

if length(refsig) == 0
    return
end

accum_ref = floor(sum(refsig));
if accum_ref <= 0.0
   mssg = sprintf('Error: ref_timing_node, ''%s'', is zero!',ref_timing_node);
   error_box(mssg,'Error using ''plot_pll_jitter_vs_time''');
   return
end

for i = length(refsig):-1:1
   if refsig(i) ~= 0.0
      final_time = time(i);
      break;
   end
end

Ts = (final_time-time(1))/accum_ref;

%%%%%%%%%%%%%%%%%%%%% Loop through subplots   %%%%%%%%%%%%%%%%%%%%%%
nsp = 1 + sum(nodes == ';');	% number of subplots
subplot(nsp,1,1);

for i = 1:nsp
    subplot(nsp,1,i);
%%%%%%%%%%%%%%%%%%%%% y definition %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    [ysig,legend_string] = create_ysignals_and_legend(x,nodes,i);
    if length(ysig) == 0
       return
    end

%   create VCO output phase based on integration of vin
    for k = 1:size(ysig,2)

       [phase,avg_period] = extract_phase(ysig(:,k),refsig);
       [edge_time] = extract_time(ysig(:,k),time);

       if avg_period <= 1.0
           mssg = sprintf('Error:  average period of refsig, ''%s'', must be > 1 time step',ref_timing_node);
           error_box(mssg,'Error using ''plot_pll_jitter_vs_time''');
           return; 
       end

       start_edge = -1;
       for j = 1:length(edge_time)
           if edge_time(j) >= start_time
               start_edge = j;
               break;
           end
       end

%%%%%%%%%%%%%%%%%%%%%% error checking %%%%%%%%%%%%%%%%%%%%%%%%%%%
       if start_edge < 1
           mssg = sprintf('Error:  start_time = ''%s'' is too high --- max time is only %5.3e',start_time_in,edge_time(length(edge_time)));
           error_box(mssg,'Error using ''plot_pll_jitter_vs_time''');
           return;
       end 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       last_phase = mean(phase(start_edge:length(phase)));
       count = 0;
       while 1
           if last_phase >= 0.75
               last_phase = last_phase - 1.0;
               count = count - 1;
           else
               break;
           end
       end
       if count == 0
           while 1
               if last_phase < -0.75
	           last_phase = last_phase + 1.0;
                   count = count + 1; 
               else
                   break;
               end
           end
       end
       if count ~= 0
	   phase = phase + count;
       end

       if k == 1
           phase_all = zeros(length(phase),size(ysig,2));
           period_all = zeros(size(ysig,2),1);
       end

%%%%%%%%%%%%%%%%%%%%%% error checking %%%%%%%%%%%%%%%%%%%%%%%%%%%
       if length(phase_all) ~= length(phase)
           mssg = sprintf('Error:  signals specified in nodes expression ''%s'' have differing number of edges',nodes);
           error_box(mssg,'Error using ''plot_pll_jitter_vs_time''');
           return;
       end
       phase_all(:,k) = phase;
       period_all(k) = avg_period;
    end
%%%%%%%%%%%%%%%%%%%%%% extraction and scaling  %%%%%%%%%%%%%%%%%%%%%%%%%%%
    rms_jitter_ui = 1e3*std(phase_all(start_edge:size(phase_all,1),:));
    rms_jitter = 1e-3*Ts*rms_jitter_ui.*period_all;
    if length(rms_jitter) ~= length(legend_string)
        mssg = sprintf('Error:  legend cell string and rms jitter vector are different sizes -- this shouldn''t happen!');
        error_box(mssg,'Error using ''plot_pll_jitter_vs_time''');
        return;
    end
    for n = 1:length(legend_string)
        [scaled_jitter_val,jitter_label] = determine_label(rms_jitter(n),jitter_limits,jitter_labels);
        legend_string(n) = cellstr(sprintf('%s: %5.2f %s(rms)',char(legend_string(n)),scaled_jitter_val,jitter_label));
    end
    legend_string_global{i} = legend_string;


    [edge_time,xlabel_name] = scale_and_label_x(edge_time,x_limits,x_labels);

    switch xlabel_name
       case 'Time (picoseconds)'
          sc_start_time = start_time*1e12;
       case 'Time (nanoseconds)'
          sc_start_time = start_time*1e9;
       case 'Time (microseconds)'
          sc_start_time = start_time*1e6;
       case 'Time (milliseconds)'
          sc_start_time = start_time*1e3;
       otherwise
          sc_start_time = start_time;
    end
%%%%%%%%%%%%%%%%%%%%% Plot function %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    deci_plot(edge_time,phase_all,i);
    hold on
       y_ax = get(gca,'YLim');
       plot([sc_start_time sc_start_time],[y_ax(1) y_ax(2)],'r-');
    hold off

%%%%%%%%%%%%%%%%%%%%% Labeling %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    h_leg = legend(legend_string);
    set(h_leg,'TextColor',[1 1 1]);
end

create_title(title_name);
create_ylabel(ylabel_name);
create_xlabel(xlabel_name); 

status = 0;
