%
%   Written by Michael Perrott for viewing CppSim/VppSim output
%   Distribution:  http://www.cppsim.com
%
%   This code is used as part of the CppSimView program
%   (cppsimview.m)
%
%   For those that use this code - please
%   keep my name attached to it (as currently done).  
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU Library General Public
% License along with this library; if not, write to the MxFree
% Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

% Plot expressions invoking signal names from structure x.
% Usage: plot_xy(x, 'xsig','nodes') 
% In nodes, commas delimit curves to be overlayed and semicolons delimit
% seperate subplots on the same figure.  E.g., plot_xy(x,'x1','v1,v2;v3')
% creates two subplots, with voltages v1 and v2 overlayed.  Also note
% that numeric node names should be prepended by '@' to distinguish
% them from constants.
%  written by Mike Perrott

function status = plot_xy(x,x_sig,nodes)

%%%%%%%%%%%%%%%%%%%%% Error checking %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin ~= 3
     mssg = sprintf('Error:  must have 3 arguments: plot_xy(x,''x_sig'',''nodes'')');
     error_box(mssg,'Error using ''plot_xy''');
     return 
end
%%%%%%%%%%%%%%%%%%%%% Labels  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


ylabel_name = ' ';
if sum(nodes == ';') > 0 | sum(nodes == ',') > 0
   title_name = 'CppSim Simulated Signals';
else
   title_name = 'CppSim Simulated Signal';
end

clf
status = -1;

%%%%%%%%%%%%%%%%%%%%% x definition %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if sum(x_sig == ',') ~= 0
     mssg = sprintf('Error:  cannot have commas in x_sig description (which is ''%s'' in this case)',x_sig);
     error_box(mssg,'Error using ''plot_xy''');
     return 
end
if sum(x_sig == ';') ~= 0
     mssg = sprintf('Error:  cannot have semicolons in x_sig description (which is ''%s'' in this case)',x_sig);
     error_box(mssg,'Error using ''plot_xy''');
     return 
end

xsig = evalsig(x,char(x_sig));
if length(xsig) == 0
    return
end

x_labels = cellstr(sprintf('%s',strrep(char(x_sig),'_','\_')));
x_limits = [1];

[xscaled,xlabel_name] = scale_and_label_x(xsig,x_limits,x_labels);

%%%%%%%%%%%%%%%%%%%%% Loop through subplots   %%%%%%%%%%%%%%%%%%%%%%
nsp = 1 + sum(nodes == ';');	% number of subplots
subplot(nsp,1,1);

for i = 1:nsp
    subplot(nsp,1,i);
%%%%%%%%%%%%%%%%%%%%% y definition %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    [ysig,legend_string] = create_ysignals_and_legend(x,nodes,i);
    if length(ysig) == 0
       return
    end

%%%%%%%%%%%%%%%%%%%%% Plot function %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    deci_plot(xscaled,ysig,i,'nodec');

%%%%%%%%%%%%%%%%%%%%% Labeling %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    h_leg = legend(legend_string);
    set(h_leg,'TextColor',[1 1 1]);
end

create_title(title_name);
create_ylabel(ylabel_name);
create_xlabel(xlabel_name); 

status = 0;
