%
%   Written by Michael Perrott for viewing CppSim/VppSim output
%   Distribution:  http://www.cppsim.com
%
%   This code is used as part of the CppSimView program
%   (cppsimview.m)
%
%   For those that use this code - please
%   keep all names (i.e., Michael Perrott, Belal Helal)
%   attached to it (as currently done).  
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU Library General Public
% License along with this library; if not, write to the MxFree
% Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function status = xzoom (Command , Command_input)
%   Written by Michael Perrott based on
%   tzoom written by Belal M Helal (bhelal@mit.edu) as part of the CppSim package.
%   For more information: http://www-mtl.mit.edu/research/perrottgroup/tools.html
%
%

status = -1;
global plot_axes_handle cur_marker min_x_global
persistent prev_min_x prev_max_x
persistent y_value_prev x_value_prev x_index_prev

hf=get(0,'currentfigure');  %get the handle for the current figure
fig_name=get(hf,'Name');    %get the figure name
if isempty(hf)              %if there is no current figure, do nothing
   return
end

if nargin==0
    Command='gui';
elseif nargin==1
    Command_input=[];
end;


Command=lower(Command);      %make sure that commands are in lower case

switch Command               %switch branching for different commands
%--------------------------------------------------------------------------
    case 'zoom',
        xx=ginput2(Command_input); % get points from figure
        if xx(1)==xx(2) %do nothing if the two points are equal
            return
        end
        if xx(1) < xx(2)
           min_x = xx(1);
           max_x = xx(2);
        else
           min_x = xx(2);
           max_x = xx(1);
        end

        prev_min_x = min_x;
        prev_max_x = max_x;

        deci_plot(min_x,max_x,0);
        plot_axes_handle = gca;

%--------------------------------------------------------------------------
    case 'prev_zoom',

        deci_plot(prev_min_x,prev_max_x,0);
        plot_axes_handle = gca;
%--------------------------------------------------------------------------
    case 'gui'

	fig_items = get(gcf,'Children');
        for i = 1:length(fig_items)
            if strcmp(get(fig_items(i),'Type'),'uicontrol') == 1
                return
            end
        end

	    % Construct GUI
	    %position format is [buttonx, buttony,buttonwidth,buttonheight]
	    hb = uicontrol(hf   ,...                 % Time Zoom Button
             'Style'          , 'pushbutton',...
             'Units'          , 'normalized',...
             'String'         , '(Z)oom X',...
             'FontName'       , 'interface user',...
             'Position'       , [.04, 0.95 ,0.11,.05],...
             'Callback'       ,'xzoom zoom;');
         

		hb = uicontrol(hf   ,...             % Full Time Button
             'Style'          , 'pushbutton',...
             'Units'          , 'normalized',...
             'String'         , '(F)ull X',...
             'FontName'       , 'interface user',...
             'Position'       , [.16, 0.95 ,0.11,.05],...
             'Callback'       ,'xzoom full;');         

		hb = uicontrol(hf   ,...             % Prev Zoom Button
             'Style'          , 'pushbutton',...
             'Units'          , 'normalized',...
             'String'         , '(P)rev Zoom',...
             'FontName'       , 'interface user',...
             'Position'       , [.28, 0.95 ,0.11,.05],...
             'Callback'       ,'xzoom prev_zoom;');         

		hb = uicontrol(hf   ,...             % Pan Left Button
             'Style'          , 'pushbutton',...
             'Units'          , 'normalized',...
             'String'         , '<',...
             'FontName'       , 'interface user',...
             'Position'       , [.40, 0.95 ,0.05,.05],...
             'Callback'       ,'xzoom pan_left;');         

		hb = uicontrol(hf   ,...             % Pan Right Button
             'Style'          , 'pushbutton',...
             'Units'          , 'normalized',...
             'String'         , '>',...
             'FontName'       , 'interface user',...
             'Position'       , [.45, 0.95 ,0.05,.05],...
             'Callback'       ,'xzoom pan_right;');         

		hb = uicontrol(hf   ,...             % Zoom Out Button
             'Style'          , 'pushbutton',...
             'Units'          , 'normalized',...
             'String'         , 'In',...
             'FontName'       , 'interface user',...
             'Position'       , [.5, 0.95 ,0.05,.05],...
             'Callback'       ,'xzoom zoom_in;');         

		hb = uicontrol(hf   ,...             % Zoom In Button
             'Style'          , 'pushbutton',...
             'Units'          , 'normalized',...
             'String'         , 'Out',...
             'FontName'       , 'interface user',...
             'Position'       , [.55, 0.95 ,0.05,.05],...
             'Callback'       ,'xzoom zoom_out;');         

		hb = uicontrol(hf   ,...             % Measure
             'Style'          , 'pushbutton',...
             'Units'          , 'normalized',...
             'String'         , '(M)easure',...
             'FontName'       , 'interface user',...
             'Position'       , [.61, 0.95 ,0.11,.05],...
             'Callback'       ,'xzoom measure;');         

		hb = uicontrol(hf   ,...             % Measure Diff
             'Style'          , 'pushbutton',...
             'Units'          , 'normalized',...
             'String'         , 'Meas(D)iff',...
             'FontName'       , 'interface user',...
             'Position'       , [.73, 0.95 ,0.11,.05],...
             'Callback'       ,'xzoom measurediff;');         

		hb = uicontrol(hf   ,...             % LineStyle
             'Style'          , 'pushbutton',...
             'Units'          , 'normalized',...
             'String'         , '(L)ineStyle',...
             'FontName'       , 'interface user',...
             'Position'       , [.85, 0.95 ,0.11,.05],...
             'Callback'       ,'xzoom marker;');         

            set(hf,'KeyPressFcn','xzoom hotkey;');       % enable hotkeys
%            deci_plot(1,1,3);
%--------------------------------------------------------------------------
    case 'remove'
        h_uic=findobj(hf,'Type','uicontrol')';
        delete(h_uic);
%        deci_plot(1,1,3);
%--------------------------------------------------------------------------
     case 'full',
        deci_plot(1,1,0);
        plot_axes_handle = gca;
%--------------------------------------------------------------------------
     case 'pan_left',
        deci_plot(1,1,-1);
        plot_axes_handle = gca;
%--------------------------------------------------------------------------
     case 'pan_right',
        deci_plot(1,1,1);
        plot_axes_handle = gca;
%--------------------------------------------------------------------------
     case 'zoom_in',
        deci_plot(1,1,-2);
        plot_axes_handle = gca;
%--------------------------------------------------------------------------
     case 'zoom_out',
        deci_plot(1,1,2);
        plot_axes_handle = gca;
%--------------------------------------------------------------------------
    case 'marker',

    ax_list = get(gcf,'Children');
    for i = 1:length(ax_list)
        if strcmp(get(ax_list(i),'Type'),'axes') == 1
           if strcmp(get(ax_list(i),'YAxisLocation'),'left') == 1 & ...
              strcmp(get(ax_list(i),'Tag'),'legend') == 0
              hli=get(ax_list(i),'Children'); % get the handle of lines in axis
            
               for i=1:length(hli)
                  cur_mark = get(hli(i),'Marker');
                  if strcmp(cur_mark,'o') == 1
                      set(hli(i),'Marker','none');
                      cur_marker = 'none';
                  else
                      set(hli(i),'Marker','o');
                      cur_marker = 'o';
                  end
              end
           end
        end
    end
%--------------------------------------------------------------------------
    case 'measure'

      first_button_flag = 0;
      while 1

        [x,y,button]=ginput(1); %get one point using the mouse

        if button == 1
           first_button_flag = 1;
        end
        if button ~= 1 & first_button_flag == 1
 	    break;
        end		

        ha=gca;
        hl=get(ha,'Children');      %get the handle of line in the last axis

        nl=length(hl);              %nl=number lines

        for i=1:nl       %loop through the lines in the axis
            xdata=get(hl(i),'XData');
            if length(xdata) > 2
	        break;
            end
        end

        % get the index of the nearest starting point
	ind=find( min( abs( xdata-x ) ) == ( abs( xdata-x ) ) ); 
        x_index=ind(1); % dealing with more than one point at same x-coordinate
        x_value=xdata(x_index);

        y_value=zeros(nl,1);        
        j = 1;
        for i=1:nl       %loop through the lines in the axis
            ydata=get(hl(i),'YData');
            if length(ydata) > 2
               y_value(j)=ydata(x_index);
               j = j+1;
            end
        end
        y_value= flipud(y_value);
        ys_value=flipud(sort(y_value));  %sort the array in desceding order

	flag = 0;
        for i=1:nl       %loop through the lines in the axis
            ydata=get(hl(i),'YData');
            if length(ydata) <= 2
 	       [y_choose,i_choose] = min(abs(ys_value-y));
               y_value = ys_value(i_choose);
 	       set(hl(i),'Ydata',y_value);
               set(hl(i),'Xdata',x_value); 
               set(hl(i),'Marker','o');
               set(hl(i),'Color',[1 0 0]);
               flag = 1;
               break;
            end
        end
        if flag == 0
           hold on;
 	   [y_choose,i_choose] = min(abs(ys_value-y));
           y_value = ys_value(i_choose);
           plot(x_value,y_value,'ro');
           hold off;
        end
	y_value_prev = y_value;
        x_value_prev = x_value;
        x_index_prev = x_index+min_x_global-2;

      end


      %%%%% check if right axis exists     
      ax_list = get(gcf,'Children');

      global right_axis_offset_global
      right_axis_offset = 0;
      for i = 1:length(ax_list)
         if strcmp(get(ax_list(i),'Type'),'axes') == 1
             if strcmp(get(ax_list(i),'YAxisLocation'),'right') == 1 & ...
                strcmp(get(ax_list(i),'Tag'),'legend') == 0
                     right_axis_offset = right_axis_offset_global;
                     break
             end
         end
      end

      fig_name=get(hf,'Name');    %get the figure name

      if right_axis_offset == 0
         mssg_box({['Figure ',fig_name];'';...
                    'Sample Number:';num2str(x_index+min_x_global-2);''; ...
                    'X-Value:'; num2str(x_value,10);''; ...
                    'Y-Value:'; num2str(y_value,10)}, ...
                    'Nearest Point Coordinates');
      else
         mssg_box({['Figure ',fig_name];'';...
                    'Sample Number:';num2str(x_index+min_x_global-2);''; ...
                    'X-Value:'; num2str(x_value,10);''; ...
                    'Y-Value (left axis):'; num2str(y_value,10);''; ...
                    'Y-Value (right axis):'; num2str(y_value+right_axis_offset,10)}, ...
                    'Point Coordinates');
      end
%--------------------------------------------------------------------------
    case 'measurediff'

      point_flag = -1;
      while 1

        [x,y,button]=ginput(1); %get one point using the mouse

        if point_flag == -1
            if button == 1
                point_flag = 0;
            else
                point_flag = 1;
            end
        end
        if button ~= 1 & point_flag == 0
	    y_value_prev = y_value;
            x_value_prev = x_value;
            x_index_prev = x_index+min_x_global-2;
  	    point_flag = 1;
        elseif button == 1 & point_flag == 1
	    break;
        end

        ha=gca;
        hl=get(ha,'Children');      %get the handle of line in the last axis

        nl=length(hl);              %nl=number lines

        for i=1:nl       %loop through the lines in the axis
            xdata=get(hl(i),'XData');
            if length(xdata) > 2
	        break;
            end
        end

        % get the index of the nearest starting point
	ind=find( min( abs( xdata-x ) ) == ( abs( xdata-x ) ) ); 
        x_index=ind(1); % dealing with more than one point at same x-coordinate
        x_value=xdata(x_index);

        y_value=zeros(nl,1);        
        j = 1;
        for i=1:nl       %loop through the lines in the axis
            ydata=get(hl(i),'YData');
            if length(ydata) > 2
               y_value(j)=ydata(x_index);
               j = j+1;
            end
        end
        y_value= flipud(y_value);
        ys_value=flipud(sort(y_value));  %sort the array in descending order

	flag = 0;
        for i=1:nl       %loop through the lines in the axis
            ydata=get(hl(i),'YData');
            if length(ydata) <= 2
 	       [y_choose,i_choose] = min(abs(ys_value-y));
               y_value = ys_value(i_choose);
               if point_flag == 0
 	          set(hl(i),'Ydata',y_value);
                  set(hl(i),'Xdata',x_value); 
                  set(hl(i),'Marker','o');
                  set(hl(i),'Color',[1 0 0]);
               else
 	          set(hl(i),'Ydata',[y_value_prev,y_value]);
                  set(hl(i),'Xdata',[x_value_prev,x_value]); 
                  set(hl(i),'Marker','o');
                  set(hl(i),'LineStyle','-');
                  set(hl(i),'Color',[1 0 0]);
               end
               flag = 1;
               break;
            end
        end
        if flag == 0 & point_flag == 0
           hold on;
 	   [y_choose,i_choose] = min(abs(ys_value-y));
           y_value = ys_value(i_choose);
           plot(x_value,y_value,'ro');
           hold off;
        elseif flag == 0 & point_flag == 1
           hold on;
 	   [y_choose,i_choose] = min(abs(ys_value-y));
           y_value = ys_value(i_choose);
           plot([x_value_prev,x_value],[y_value_prev,y_value],'ro-');
           hold off;
        end

      end

      fig_name=get(hf,'Name');    %get the figure name

      if x_value ~= x_value_prev
           slope=(y_value-y_value_prev)/(x_value-x_value_prev);
      else
           slope=0;
      end

      x_index = x_index+min_x_global-2;
      
      mssg_box({['Figure ',fig_name];''; ...
        'Delta Sample Number:';num2str(x_index-x_index_prev,20); ...
        '';'Delta Time:'; num2str(x_value-x_value_prev,10);''; ...
        'Delta Y-Value:';num2str(y_value-y_value_prev,10) ;''; ...
	'Slope:';num2str(slope,10) },'Difference in Point Coordinates');

%--------------------------------------------------------------------------
    case 'hotkey'

        cur_fig = gcf;
        if strcmp(get(cur_fig,'Type'),'figure') == 0
            return
        end
        hkey=get(cur_fig,'CurrentCharacter');
        switch hkey         %you can modify the hotkeys below
            case {'z','Z'}
                xzoom zoom;
            case {'f','F'}
                xzoom full;
            case {'l','L'}
                xzoom marker;
            case {'m','M'}
                xzoom measure;
            case {'p','P'}
                xzoom prev_zoom;
            case {'d','D'}
                xzoom measurediff;
            otherwise
                if int2str(hkey) == '28' | hkey == '['   %%% left arrow
                    xzoom('pan_left');
                elseif int2str(hkey) == '29' | hkey == ']' %%% right arrow
                    xzoom('pan_right');
                elseif int2str(hkey) == '30'    %%% zoom out
                    xzoom('zoom_out');
                elseif int2str(hkey) == '31'    %%% zoom in
                    xzoom('zoom_in');
                else
		    return
                end
        end  % end of switch statment
    
    end  % end of switch statment

status = 0;

%--------------------------------------------------------------------------
function two_points=ginput2(com_inp)
        if isempty(com_inp) %if no arguments are used, then get the two points by mouse
            [x,y,button]=ginput(2);                     %get the two time points from the figure using the mouse
            if sum(button==1)~=2;x=[0 0];end;           %if the buttons pressed are not both the left buttons, flag to do nothing
        else
            x=com_inp;
        end
two_points=[min(x) max(x)];                 %order the two points


