#### Procedures for CppSim Simulation

# Edit Sim File
proc cppsim_edit_sim_file {schematic_wt} {

global CPPSIM_HOME CPPSIMSHARED_HOME EDITOR CUR_CPPSIM_SIM_FILE

set title_list [split [db::getAttr title -of $schematic_wt] " "]
set TOP_CELL [lrange $title_list 2 2]
set TOP_LIBRARY [lrange $title_list 1 1]

set cur_dir "$CPPSIM_HOME/SimRuns"
if {[file isdirectory $cur_dir] == 0} {
    if {[catch {file mkdir $cur_dir} message]} {
      hspc_error_dialog "Error in creating SimRuns directory: $cur_dir" $schematic_wt
      return
    }
}
set cur_dir "$CPPSIM_HOME/SimRuns/$TOP_LIBRARY"
if {[file isdirectory $cur_dir] == 0} {
    if {[catch {file mkdir $cur_dir} message]} {
      hspc_error_dialog "Error in creating SimRuns directory: $cur_dir" $schematic_wt
      return
    }
}
set cur_dir "$CPPSIM_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL"
if {[file isdirectory $cur_dir] == 0} {
    if {[catch {file mkdir $cur_dir} message]} {
      hspc_error_dialog "Error in creating SimRuns directory: $cur_dir" $schematic_wt
      return
    }
}


if {[file isfile "$CPPSIM_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL/test.par"] == 0} {

    set CUR_CPPSIM_SIM_FILE test.par
    set cur_file "$cur_dir/$CUR_CPPSIM_SIM_FILE"
    set base_name [string trimright $CUR_CPPSIM_SIM_FILE "par"]
    set base_name [string trimright $base_name "."]

    ####### Open sim file
    if {[catch "set fid \[open $cur_file w\]" err]} {
      hspc_error_dialog "Error in creating Sim File: $cur_file" $schematic_wt
      return
    }

    puts $fid "/////////////////////////////////////////////////////////////"
    puts $fid "// CppSim Sim File: $CUR_CPPSIM_SIM_FILE"
    puts $fid "// Cell: $TOP_CELL"
    puts $fid "// Library: $TOP_LIBRARY"
    puts $fid "/////////////////////////////////////////////////////////////\n"
    puts $fid "// Number of simulation time steps"
    puts $fid "// Example: num_sim_steps: 10e3"
    puts $fid "num_sim_steps: \n"
    puts $fid "// Time step of simulator (in seconds)"
    puts $fid "// Example: Ts: 1/10e9"
    puts $fid "Ts: \n"
    puts $fid "// Output File name"
    puts $fid "// Example:  name below produces $base_name.tr0, $base_name.tr1, ..."
    puts $fid "// Note: you can decimate, start saving at a given time offset, etc."
    puts $fid "//    -> See pages 34-35 of CppSim manual (i.e., output: section)"
    puts $fid "output: $base_name \n"
    puts $fid "// Nodes to be included in Output File"
    puts $fid "// Example: probe: n0 n1 xi12.n3 xi14.xi12.n0"
    puts $fid "probe: \n"
    puts $fid "/////////////////////////////////////////////////////////////"
    puts $fid "// Note:  Items below can be kept blank if desired"
    puts $fid "/////////////////////////////////////////////////////////////\n"
    puts $fid "// Values for global parameters used in schematic"
    puts $fid "// Example: global_param: in_gl=92.1 delta_gl=0.0 step_time_gl=100e3*Ts"
    puts $fid "global_param: \n"

    puts $fid "// Rerun simulation with different global parameter values"
    puts $fid "// Example: alter: in_gl = 90:2:98"
    puts $fid "// See pages 37-38 of CppSim manual (i.e., alter: section)"
    puts $fid "alter: "


    if {[catch {close $fid} err]} {
      hspc_error_dialog "Error in closing Sim File: $cur_file" $schematic_wt
      return
    }

}

set cur_file "$cur_dir/$CUR_CPPSIM_SIM_FILE"

####### Open sim file
if {[catch "exec $EDITOR -bg black -fg yellow -T $cur_file $cur_file &" err]} {
      hspc_error_dialog "Error in editing Sim File: $cur_file" $schematic_wt
      return
}


}



# Synchronize to Sue2 schematic
proc cppsim_choose_sim_file {schematic_wt} {

global CPPSIM_HOME CPPSIMSHARED_HOME EDITOR CUR_CPPSIM_SIM_FILE

set title_list [split [db::getAttr title -of $schematic_wt] " "]
set TOP_CELL [lrange $title_list 2 2]
set TOP_LIBRARY [lrange $title_list 1 1]

set cur_dir "$CPPSIM_HOME/SimRuns"
if {[file isdirectory $cur_dir] == 0} {
    if {[catch {file mkdir $cur_dir} message]} {
      hspc_error_dialog "Error in creating SimRuns directory: $cur_dir" $schematic_wt
      return
    }
}
set cur_dir "$CPPSIM_HOME/SimRuns/$TOP_LIBRARY"
if {[file isdirectory $cur_dir] == 0} {
    if {[catch {file mkdir $cur_dir} message]} {
      hspc_error_dialog "Error in creating SimRuns directory: $cur_dir" $schematic_wt
      return
    }
}
set cur_dir "$CPPSIM_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL"
if {[file isdirectory $cur_dir] == 0} {
    if {[catch {file mkdir $cur_dir} message]} {
      hspc_error_dialog "Error in creating SimRuns directory: $cur_dir" $schematic_wt
      return
    }
}

if {[catch {glob $cur_dir/*.par} message] != 1} {
#   $SIM_FILE_MENU delete 0 end
   set count 0
   set file_name_list ""
   foreach file [glob $cur_dir/*.par] {
#      puts "file: $file"
      set i [string last "/" $file]
       set base_filename [string range $file [expr $i + 1] end]
      lappend file_name_list $base_filename
      set count [expr $count + 1]
   }
} else {
   set CUR_CPPSIM_SIM_FILE "test.par"
   set count 0
}

if {$count > 0} {
   set simFileRadioContainer [gi::createDialog cppsimChooseSimRadioContainer \
                                -title "Choose Sim File" \
                                -showHelp false -execProc hspc_do_nothing \
				-parent $schematic_wt]
   set simFileRadio [gi::createMutexInput cppsimChooseSimRadio  \
		      -shown true -parent $simFileRadioContainer \
		      -viewType radio -enum $file_name_list \
			 -label "Possible Sim File Choices" -radioRows $count -value "test.par" \
		      -valueChangeProc cppsim_set_sim_file_name]
} else {
      hspc_error_dialog "Error:  No Sim Files are found.  Please click on 'Edit Sim File' to create one." \
	  $schematic_wt
      return
}

}


proc cppsim_set_sim_file_name {w} {
    global CUR_CPPSIM_SIM_FILE CPPSIM_SIM_MENU

    set CUR_CPPSIM_SIM_FILE [db::getAttr value -of $w]
#    set menu_attr [db::getAttr title -of $CPPSIM_SIM_MENU]
    if {[string compare $CUR_CPPSIM_SIM_FILE "test.par"] == 0} {
         db::setAttr title -of $CPPSIM_SIM_MENU -value "CppSim"
    } else {
         db::setAttr title -of $CPPSIM_SIM_MENU -value "CppSim: $CUR_CPPSIM_SIM_FILE"
    }
}


#########################################################################################################

proc cppsim_start_custom_waveview {schematic_wt} {

global CPPSIMSHARED_HOME CPPSIM_HOME CUR_SIM_FILE

set title_list [split [db::getAttr title -of $schematic_wt] " "]
set TOP_CELL [lrange $title_list 2 2]
set TOP_LIBRARY [lrange $title_list 1 1]
set run_dir "$CPPSIM_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL"

if {[file isfile "$run_dir/test.tr0"] == 0} {
    hspc_error_dialog "Error: you must run CppSim (and probe signals) before starting Custom WaveView (Transient)" $schematic_wt
    puts "Error: you must run CppSim (and probe signals) before starting Custom WaveView (Transient)"
    return
} 

xt::createJob custom_waveview_tr -type batch -cmdLine "cd $run_dir; sx -w -k -load test.tr0" -runDesc "Custom WaveView (CppSim)"

}

#########################################################################################################

# Netlist and run CppSim routine
proc cppsim_netlist_and_run {schematic_wt vppsim_mode} {

global CPPSIMSHARED_HOME CPPSIM_HOME CUR_SIM_FILE CPPSIM_RUN_FLAG CPPSIM_JOB_ID

if {$CPPSIM_RUN_FLAG == 1} {
    hspc_error_dialog "Error: Cppsim or VppSim already running (click on 'Kill Cppsim/VppSim Run' to terminate)" $schematic_wt
    return
}

if {[auto_execok net2code] == ""} {
   hspc_error_dialog "Error: Cppsim/VppSim net2code command not found!" $schematic_wt
   puts "Error:  net2code command not found!"
   puts "      -> make sure that you have installed Cppsim and"
   puts "         included it in your path"
   if {$vppsim_mode == 0} {
       puts "------------------ Error: running cppsim failed --------------------"
   } else {
       puts "------------------ Error: running vppsim failed --------------------"
   }
   return
}

set title_list [split [db::getAttr title -of $schematic_wt] " "]
set TOP_CELL [lrange $title_list 2 2]
set TOP_LIBRARY [lrange $title_list 1 1]


if {[file isfile "$CPPSIM_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL/test.par"] == 0} {
    hspc_error_dialog "Error: you must create a Sim file (click on 'Edit Sim File')" $schematic_wt
    puts "Error: must create a Sim file (click on 'Edit Sim File')"
   if {$vppsim_mode == 0} {
       puts "------------------ Error: running cppsim failed --------------------"
   } else {
       puts "------------------ Error: running vppsim failed --------------------"
   }
    return
} 

set CPPSIM_RUN_FLAG 1

set return_val [cppsim_netlist_only $schematic_wt "lite"]

if {$return_val == 1} {
   hspc_error_dialog "Error:  CppSim/VppSim netlisting failed" $schematic_wt
   puts "*** Please Fix the Netlisting Issue ***"
   set CPPSIM_RUN_FLAG 0
   return
} 

set netlist_dir "$CPPSIM_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL"

puts "-------------- running net2code program --------------"

catch {cd $netlist_dir}
if {$vppsim_mode == 0} {
   catch {open "|net2code -cpp |& cat"} fid
} else {
   catch {open "|net2code -vpp |& cat"} fid
}
set message_input [read $fid]
puts $message_input
if {[catch {close $fid} err] != 0} {
    hspc_error_dialog "Error: net2code command failed!" $schematic_wt
    puts "------------------ Net2code Program Exited with Errors --------------------"
    set CPPSIM_RUN_FLAG 0
    return
}

if {[file isfile "$netlist_dir/Makefile"] == 0} {
   if {$vppsim_mode == 0} {
       hspc_error_dialog "Error: Cppsim Makefile not found!" $schematic_wt
   } else {
       hspc_error_dialog "Error: Vppsim Makefile not found!" $schematic_wt
   }

   puts "Error:  Makefile is missing (should be produced by net2code program)!"
   puts "      (i.e., can't open file '$netlist_dir/Makefile')"
   puts "------------------ Error: running net2code failed --------------------"
   set CPPSIM_RUN_FLAG 0
   return
}

if {$vppsim_mode == 0} {
    puts "--- running Cppsim (Ouput directory: $CPPSIM_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL) ---"
    set CPPSIM_JOB_ID [xt::createJob cppsim -type batch -cmdLine "make" -runDesc "Cppsim simulation of module '$TOP_CELL' (lib '$TOP_LIBRARY')" -exitProc cppsim_message]
} else {
    puts "--- running Vppsim (Ouput directory: $CPPSIM_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL) ---"
    set CPPSIM_JOB_ID [xt::createJob cppsim -type batch -cmdLine "make" -runDesc "Vppsim simulation of module '$TOP_CELL' (lib '$TOP_LIBRARY')" -exitProc cppsim_message]
}



}

proc cppsim_add_conversion_module_to_netlist {libName cellName fid_netlist} {

set dmCellView [dm::getCellViews schematic -cellName $cellName -libName $libName]
if {[db::isEmpty $dmCellView]} {
    de::sendMessage "Error: conversion module '$cellName' (lib '$libName') schematic doesn't exist" -severity error
    return 1
}
if { ![catch {set context [de::open $dmCellView -headless true -readOnly true]} result ]} {
    set conversion_module [db::getAttr topDesign -of $context]
    # puts "---> descend into module [db::getAttr cellName -of $conversion_module]"
    set return_val [cppsim_descend_into_module $conversion_module $fid_netlist]
    # puts "--------> returned! --------"
    if {$return_val == 1} {
        puts "Error: cannot descend into conversion module '$cellName' (lib '$libName')"
        return 1
    }
} else {
    puts "Error: cannot open context of conversion module '$cellName' (lib '$libName')"
    return 1
}
return 0
}

proc cppsim_netlist_only {schematic_wt lite_flag} {

    global CPPSIM_HOME CPPSIMSHARED_HOME CUR_CPPSIM_SIM_FILE

    set top_module [db::getAttr editDesign -of [de::getActiveContext]]
    set TOP_CELL [db::getAttr cellName -of $top_module]
    set TOP_LIBRARY [db::getAttr libName -of $top_module]

    puts "--------- running CppSim netlister for '$TOP_CELL' (lib '$TOP_LIBRARY')  --------"

    set netlist_dir "$CPPSIM_HOME/Netlist"
    set netlist_filename "$netlist_dir/netlist.cppsim"

    if {[file isdirectory $netlist_dir] == 0} {
       hspc_error_dialog "Error:  You must create CppSim netlist directory: $netlist_dir" $schematic_wt
       puts "--- Netlisting Exited with Errors ---"
       return 1
    }

    if {[file isfile $netlist_filename] != 0} {
        if {[catch {file delete $netlist_filename} mssg]} {
           hspc_error_dialog "Cannot delete netlist file: $netlist_filename" $schematic_wt
           puts "--- Netlisting Exited with Errors ---"
           return 1
        } 
    }

    ####### Open netlist file
    if {[catch "set fid_netlist \[open $netlist_filename a\]" err]} {
       puts "Error: cannot create netlist file: $netlist_filename"
       return 1
    }

    puts $fid_netlist "***** CppSim Netlist for Cell '$TOP_CELL' *****"
    puts $fid_netlist " "
    set return_val [cppsim_descend_into_module $top_module $fid_netlist]

    ################### Add conversion modules to netlist ###################
    set return_val [cppsim_add_conversion_module_to_netlist "CppSimModules" "cpp_internal_double_interp_to_bool_convert" $fid_netlist]
    if {$return_val == 1} {
       return 1
    }
    set return_val [cppsim_add_conversion_module_to_netlist "CppSimModules" "cpp_internal_double_to_int_convert" $fid_netlist]
    if {$return_val == 1} {
       return 1
    }
    set return_val [cppsim_add_conversion_module_to_netlist "CppSimModules" "cpp_internal_int_to_double_convert" $fid_netlist]
    if {$return_val == 1} {
       return 1
    }
    set return_val [cppsim_add_conversion_module_to_netlist "CppSimModules" "cpp_internal_bool_to_double_interp_convert" $fid_netlist]
    if {$return_val == 1} {
       return 1
    }
    ################ End of Adding Conversion Modules ################

    if {[catch {close $fid_netlist} err]} {
        puts "Error: cannot close netlist file: $netlist_filename"
        return 1
    }

    if {$return_val == 1} {
       hspc_error_dialog "Error:  CppSim netlisting failed" $schematic_wt
       puts "--- Netlisting Exited with Errors ---"
       return 1
    } 

    if {$lite_flag != "lite"} {
        hspc_operation_success_message_dialog "CppSim netlisting completed!" $schematic_wt
    }
     puts "--- CppSim netlisting successfully completed ---"

    if {$lite_flag != "lite"} {
       if {[file isfile "$CPPSIM_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL/test.par"] == 0} {
          puts "*** Please create a Sim file (click on 'Edit Sim File') ***"
       } else {
          puts "*** Note:  to run CppSim from Linux shell: ***"
          puts "> cd $CPPSIM_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL"
          if {$CUR_CPPSIM_SIM_FILE == "test.par"} {
             puts "> net2code -cpp"
          } else {
             puts "> net2code -cpp $CUR_CPPSIM_SIM_FILE"
          }
          puts "> make"
          puts "*** Note:  to run CppSim within Matlab: ***"
          puts "> addpath('$CPPSIMSHARED_HOME/HspiceToolbox');"
          puts "> cd $CPPSIM_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL"
          if {$CUR_CPPSIM_SIM_FILE == "test.par"} {
             puts "> cppsim_cdesigner_nonet"
          } else {
             puts "> cppsim_cdesigner_nonet('$CUR_CPPSIM_SIM_FILE')"
          }
      }
   }
return 0
}

proc cppsim_descend_into_module {cur_module fid_netlist} {
    set module_name [db::getAttr cellName -of $cur_module]
    set lib_name [db::getAttr libName -of $cur_module]

    set cur_lib [db::getAttr lib -of $cur_module]
    set cur_lib_path [db::getAttr fullPath -of $cur_lib]
    set cur_module_path "$cur_lib_path/$module_name/schematic/sch.oa"

#   puts "Entered module '$module_name'"
    puts $fid_netlist "************** Module $module_name *****************"
    puts $fid_netlist "module: $module_name $lib_name"
    puts $fid_netlist "path: $cur_module_path"

    db::foreach cur_term [db::getTerms -of $cur_module] {
        set term_name [db::getAttr name -of $cur_term]
        set term_net [db::getAttr net -of $cur_term]
        set term_net_name [db::getAttr name -of $term_net]
        set term_type [db::getAttr termType -of $cur_term]
	if {$term_name != $term_net_name} {
           puts "Error in netlisting module '$module_name' (lib '$lib_name')"
           puts "   module terminal '$term_name' is connected a net with"
           puts "   a different name of '$term_net_name'"
           puts "   -> Please correct your schematic for this module"
           hspc_error_dialog "Error: netlisting not completed due to terminal/net name mismatch" [gi::getActiveWindow]
           return 1
	}
        puts $fid_netlist "   module_terminal: $term_name $term_type"
    }

    db::foreach cur_inst [db::getInsts -of $cur_module] {
         if {[db::viewExists schematic -of $cur_inst] || [db::viewExists cppsim -of $cur_inst] || [db::viewExists verilog -of $cur_inst] || [db::viewExists functional -of $cur_inst] || [db::viewExists hspice -of $cur_inst] || [db::viewExists spectre -of $cur_inst]} {
	   set inst_name [db::getAttr name -of $cur_inst]
           set inst_module_name [db::getAttr cellName -of $cur_inst]
           set inst_module_lib_name [db::getAttr libName -of $cur_inst]
           puts $fid_netlist "instance: $inst_name $inst_module_name $inst_module_lib_name"
           db::foreach cur_term [db::getInstTerms -of $cur_inst] {
              set term_name [db::getAttr name -of $cur_term]
              set term_net [db::getAttr net -of $cur_term]
              set term_net_name [db::getAttr name -of $term_net]
              puts $fid_netlist "   instance_terminal: $term_name $term_net_name"
           }
           db::foreach cur_param [db::getParams -of $cur_inst] {
              set param_def [db::getAttr paramDef -of $cur_param]
	      set param_name [db::getAttr name -of $param_def]
	      set param_value [db::getAttr value -of $cur_param]
#	      set param_def_value [db::getAttr defValue -of $param_def]
#             puts "   instance_parameter: $param_name $param_value $param_def_value"
	      if {$param_name != "lxComponentType"} {
                 puts $fid_netlist "   instance_parameter: $param_name $param_value"
              }
           }
	 }
    }
    db::foreach cur_inst [db::getInsts -of $cur_module] {
         set inst_module_name [db::getAttr cellName -of $cur_inst]
         set inst_module_lib_name [db::getAttr libName -of $cur_inst]
         if {[db::viewExists schematic -of $cur_inst]} {
           set dmCellView [dm::getCellViews schematic -cellName $inst_module_name -libName $inst_module_lib_name]
           if {[db::isEmpty $dmCellView]} {
               de::sendMessage "Error: module '$cellName' (lib '$libName') schematic doesn't exist" -severity error
               return 1
           }
           if { ![catch {set context [de::open $dmCellView -headless true -readOnly true]} result ]} {
               set inst_module [db::getAttr topDesign -of $context]
	      # puts "---> descend into module [db::getAttr cellName -of $inst_module]"
	      set return_val [cppsim_descend_into_module $inst_module $fid_netlist]
              # puts "--------> returned! --------"
              if {$return_val == 1} {
                  return 1
	      }
	   } else {
               return 1
	   }
	 } elseif {[db::viewExists cppsim -of $cur_inst] || [db::viewExists verilog -of $cur_inst] || [db::viewExists functional -of $cur_inst] || [db::viewExists hspice -of $cur_inst] || [db::viewExists spectre -of $cur_inst]} {
              set inst_module [db::getAttr master -of $cur_inst]
	      # puts "---> descend into module [db::getAttr cellName -of $inst_module]"
	      set return_val [cppsim_descend_into_module $inst_module $fid_netlist]
              # puts "--------> returned! --------"
              if {$return_val == 1} {
                  return 1
	      }
	 }
    }
return 0
}

# Read and log output from the program
proc cppsim_message {job_obj} {

global CPPSIM_RUN_FLAG

set job_status [db::getAttr exitStatus -of $job_obj]
set run_desc [db::getAttr runDesc -of $job_obj]

if {$job_status == 0} {
   hspc_operation_success_message_dialog "$run_desc successfully completed" [gi::getActiveWindow]
   puts "--- $run_desc successfully completed  ---"
} else {
    if {$CPPSIM_RUN_FLAG == 0} {
      hspc_operation_success_message_dialog "Killed $run_desc!" [gi::getActiveWindow]
      puts "*** Killed $run_desc ***"
    } else {
      hspc_error_dialog "Error: $run_desc FAILED!" [gi::getActiveWindow]
      puts "*** Error: $run_desc FAILED! ***"
    }
}
set CPPSIM_RUN_FLAG 0

# puts "*** finished hspice with status $job_status"
}

# Kill current CppSim run

proc cppsim_kill_run {schematic_wt} {

global CPPSIM_RUN_FLAG CPPSIM_JOB_ID

if {$CPPSIM_RUN_FLAG == 1} {
    xt::signalJob $CPPSIM_JOB_ID -signal kill
    set CPPSIM_RUN_FLAG 0
} else {
    hspc_error_dialog "Error: no CppSim job to kill" $schematic_wt
}

}

#####################################################################################

# Netlist only routine based on Hspice - no longer used
proc cppsim_netlist_only_hspice_based {schematic_wt lite_flag} {


global CPPSIMSHARED_HOME CPPSIM_HOME CUR_CPPSIM_SIM_FILE SYNOPSYS_LIB_FILE_DIR

set title_list [split [db::getAttr title -of $schematic_wt] " "]
set TOP_CELL [lrange $title_list 2 2]
set TOP_LIBRARY [lrange $title_list 1 1]

puts "--------- running hspice netlister for '$TOP_CELL' (Lib '$TOP_LIBRARY')  --------"

set netlist_dir "$CPPSIM_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL"

if {[file isdirectory $netlist_dir] == 0} {
   hspc_error_dialog "No Netlist Directory!" $schematic_wt
   puts "--- Netlisting Exited with Errors ---"
   return
}

if {[file isfile "$netlist_dir/netlist.hsp"] != 0} {
    if {[catch {file delete "$netlist_dir/netlist.hsp"} mssg]} {
        hspc_error_dialog "Cannot delete netlist file: $netlist_dir/netlist.hsp" $schematic_wt
        puts "--- Netlisting Exited with Errors ---"
        return
    } 
}

set netlist_obj [nl::getNetlisters HSPICE]
if {$netlist_obj eq ""} {
     hspc_error_dialog "Error: no Hspice netlister found!" $schematic_wt
     puts "--- Netlisting Exited with Errors ---"
     return   
}

db::setAttr netlistEndProc -of $netlist_obj -value "hspc_netlistEndProc"

set cv [dm::findCellView "schematic" -cellName $TOP_CELL -libName $TOP_LIBRARY]
set netlist_result [nl::runNetlister $netlist_obj -cellView $cv -filePath "$netlist_dir/netlist.hsp" \
			-viewSearchList "schematic hspiceD symbol" -viewStopList "hspiceD symbol"]

if {$netlist_result == 0} {
     hspc_error_dialog "Error: Hspice netlisting FAILED!" $schematic_wt
     puts "--- Netlisting Exited with Errors ---"
     return   
} 

# puts "--> Netlist location:  $CPPSIM_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL/netlist.hsp"

puts "--------- running spice2cppnet  --------"

set spice2cppnet_command "$CPPSIMSHARED_HOME/HspiceToolbox/Spice2CppNet/bin/spice2cppnet"
if {[auto_execok $spice2cppnet_command] == ""} {
   hspc_error_dialog "Error: spice2cppnet command not found!" $schematic_wt
   puts "Error:  spice2cppnet command not found!"
   puts "      -> make sure that you have installed the CppSim framework"
   puts "         spice2cppnet assumed location: $spice2cppnet_command"
   puts "------------------ Error: running spice2cppnet failed --------------------"
   return
}

catch {open "|$spice2cppnet_command $netlist_dir/netlist.hsp $SYNOPSYS_LIB_FILE_DIR/lib.defs \
        $CPPSIM_HOME/Netlist/netlist.cppsim |& cat"} fid
set message_input [read $fid]
puts $message_input
if {[catch {close $fid} err] != 0} {
    hspc_error_dialog "Error: spice2cppnet command failed!" $schematic_wt
    puts "------------------ Spice2CppNet Exited with Errors --------------------"
    return
} else {
    if {$lite_flag != "lite"} {
        hspc_operation_success_message_dialog "CppSim netlisting completed!" $schematic_wt
    }
     puts "--- Netlisting successfully completed ---"
}

if {$lite_flag != "lite"} {
    if {[file isfile "$CPPSIM_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL/test.par"] == 0} {
       puts "*** Please create a Sim file (click on 'Edit Sim File') ***"
   } else {
       puts "*** Note:  to run CppSim from Linux shell: ***"
       puts "> cd $CPPSIM_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL"
       if {$CUR_CPPSIM_SIM_FILE == "test.par"} {
          puts "> net2code -cpp"
       } else {
          puts "> net2code -cpp $CUR_CPPSIM_SIM_FILE"
       }
       puts "> make"
       puts "*** Note:  to run CppSim within Matlab: ***"
       puts "> addpath('$CPPSIMSHARED_HOME/HspiceToolbox');"
       puts "> cd $CPPSIM_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL"
       if {$CUR_CPPSIM_SIM_FILE == "test.par"} {
          puts "> cppsim_cdesigner_nonet"
       } else {
          puts "> cppsim_cdesigner_nonet('$CUR_CPPSIM_SIM_FILE')"
       }
   }
}
}
