#### Procedures for Hspice Simulation with HSPC


# Edit Sim File
proc hspc_edit_sim_file {schematic_wt} {

global HSPC_HOME CPPSIMSHARED_HOME EDITOR CUR_SIM_FILE HSPICE_MODEL_FILE

set title_list [split [db::getAttr title -of $schematic_wt] " "]
set TOP_CELL [lrange $title_list 2 2]
set TOP_LIBRARY [lrange $title_list 1 1]

set cur_dir "$HSPC_HOME/SimRuns"
if {[file isdirectory $cur_dir] == 0} {
    if {[catch {file mkdir $cur_dir} message]} {
      hspc_error_dialog "Error in creating SimRuns directory: $cur_dir" $schematic_wt
      return
    }
}
set cur_dir "$HSPC_HOME/SimRuns/$TOP_LIBRARY"
if {[file isdirectory $cur_dir] == 0} {
    if {[catch {file mkdir $cur_dir} message]} {
      hspc_error_dialog "Error in creating SimRuns directory: $cur_dir" $schematic_wt
      return
    }
}
set cur_dir "$HSPC_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL"
if {[file isdirectory $cur_dir] == 0} {
    if {[catch {file mkdir $cur_dir} message]} {
      hspc_error_dialog "Error in creating SimRuns directory: $cur_dir" $schematic_wt
      return
    }
}


if {[file isfile "$HSPC_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL/test.hspc"] == 0} {

    set CUR_SIM_FILE test.hspc
    set cur_file "$cur_dir/$CUR_SIM_FILE"
#    set base_name [string trimright $CUR_SIM_FILE "hspc"]
#    set base_name [string trimright $base_name "."]

    ####### Open sim file
    if {[catch "set fid \[open $cur_file w\]" err]} {
      hspc_error_dialog "Error in creating Sim File: $cur_file" $schematic_wt
      return
    }

    puts $fid "** Hspice Simulation of '$TOP_CELL' (Lib: $TOP_LIBRARY)\n"

    puts $fid "***** Note:   to understand non-Hspice commands (with '>' before them),"
    puts $fid "*****         look at HSPC manual at http://www.cppsim.com/manuals\n"

    puts $fid "***** Parameters for Calculation of Diffusion Regions for CMOS Process"
    puts $fid "> set_mode_diff geo"
    puts $fid "> set_hdout .66u"
    puts $fid "> set_hdin .74u"
    puts $fid "> use_four_sided_perimeter\n"

    puts $fid "***** Transistor Model File"
    puts $fid "***** Specify appropriate model files below"
    puts $fid ".prot"
    puts $fid ".lib '$HSPICE_MODEL_FILE' tt"
    puts $fid ".lib '$HSPICE_MODEL_FILE' tt_rfesd"
    puts $fid ".unprot"

    puts $fid "***** Temperature"
    puts $fid ".temp 25\n"

    puts $fid "***** Parameters"
    puts $fid ".param par1fn_mc=0"
    puts $fid ".param vsupply=1.5\n"

    puts $fid "***** DC Voltage Sources"
    puts $fid "Vsup avdd 0 'vsupply'"
    puts $fid "Vgnd avss 0 0.0"
    puts $fid "** Vin in 0 0.7\n"

    puts $fid "***** Simulation Options - Note match delmax to time step in .tran"
    puts $fid ".options post=1 probe=1 delmax=10n relv=1e-6 reli=1e-6 relmos=1e-6 method=gear\n"

    puts $fid "***** Simulation Parameters (.tran or .ac or .dc statements)"
    puts $fid "** .tran 10n 40u sweep Vsup 0.5 'vsupply' 0.1"
    puts $fid ".tran 10n 4u"
    puts $fid "** .dc Vsup 0 'vsupply' 0.1 sweep Vin 0 'vsupply' 0.2"
    puts $fid "** .dc Vsup 0 'vsupply' 0.1"
    puts $fid "** .ac lin 1000 100k 10Meg sweep Vsup 0.7 'vsupply' 0.1"
    puts $fid "** .ac lin 1000 100k 10Meg\n"

    puts $fid "***** Have operating point information sent to output file"
    puts $fid ".op\n"

    puts $fid "****** Digital Input Stimulus (remove lines below if you don't want this)"
    puts $fid "*** Timing Statement:  % timing delay rise/fall_time period vlow vhigh"
    puts $fid "* > timing 0.0n .2n \[1/1e9] 0 vsupply"
    puts $fid "*** Digital Input:  % input nodename \[set 0 1 1 0 1 ... R]"
    puts $fid "* > input in \[set 0 0 1 1 1 0 1 1 0 0 0 1 0 R]\n"

    puts $fid "****** Analog Input Stimulus"
    puts $fid "*** Simply add voltage or current sources as appropriate\n"

    puts $fid "****************** Selectively Probe Signals **********************"
    puts $fid "** For Hspice MOSFET models, special notation for various parameters:"
    puts $fid "** i1(m1): drain current of m1, i2(m1): gate current, i3(m1) source current"
    puts $fid "** lx7: gm0, lx8: gds, lx84: cgs, lx87: cds, lx82: total gate cap"
    puts $fid "** Click on 'Hspice MOSFET Parameter Names for .probe' for more parameter names"
    puts $fid ".probe in out i(vsup) lx7(m1) lx8(m1) lx84(m1) lx87(m1) lx82(m1)"

    if {[catch {close $fid} err]} {
      hspc_error_dialog "Error in closing Sim File: $cur_file" $schematic_wt
      return
    }

    gi::createAction "HspEditSimFile" \
        -title "Edit Sim File: Updated!" \
        -command {hspc_edit_sim_file [gi::getActiveWindow]}
}

set cur_file "$cur_dir/$CUR_SIM_FILE"

####### Open sim file
if {[catch "exec $EDITOR -bg black -fg yellow -T $cur_file $cur_file &" err]} {
      hspc_error_dialog "Error in editing Sim File: $cur_file" $schematic_wt
      return
}


}



# Synchronize to Sue2 schematic
proc hspc_choose_sim_file {schematic_wt} {

global HSPC_HOME CPPSIMSHARED_HOME EDITOR CUR_SIM_FILE

set title_list [split [db::getAttr title -of $schematic_wt] " "]
set TOP_CELL [lrange $title_list 2 2]
set TOP_LIBRARY [lrange $title_list 1 1]

set cur_dir "$HSPC_HOME/SimRuns"
if {[file isdirectory $cur_dir] == 0} {
    if {[catch {file mkdir $cur_dir} message]} {
      hspc_error_dialog "Error in creating SimRuns directory: $cur_dir" $schematic_wt
      return
    }
}
set cur_dir "$HSPC_HOME/SimRuns/$TOP_LIBRARY"
if {[file isdirectory $cur_dir] == 0} {
    if {[catch {file mkdir $cur_dir} message]} {
      hspc_error_dialog "Error in creating SimRuns directory: $cur_dir" $schematic_wt
      return
    }
}
set cur_dir "$HSPC_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL"
if {[file isdirectory $cur_dir] == 0} {
    if {[catch {file mkdir $cur_dir} message]} {
      hspc_error_dialog "Error in creating SimRuns directory: $cur_dir" $schematic_wt
      return
    }
}

if {[catch {glob $cur_dir/*.hspc} message] != 1} {
#   $SIM_FILE_MENU delete 0 end
   set count 0
   set file_name_list ""
   foreach file [glob $cur_dir/*.hspc] {
#      puts "file: $file"
      set i [string last "/" $file]
       set base_filename [string range $file [expr $i + 1] end]
      lappend file_name_list $base_filename
      set count [expr $count + 1]
   }
} else {
   set CUR_SIM_FILE "test.hspc"
   set count 0
}

if {$count > 0} {
   set simFileRadioContainer [gi::createDialog hspChooseSimRadioContainer \
                                -title "Choose Sim File" \
                                -showHelp false -execProc hspc_do_nothing \
				-parent $schematic_wt]
   set simFileRadio [gi::createMutexInput hspChooseSimRadio  \
		      -shown true -parent $simFileRadioContainer \
		      -viewType radio -enum $file_name_list \
			 -label "Possible Sim File Choices" -radioRows $count -value "test.hspc" \
		      -valueChangeProc hspc_set_sim_file_name]
} else {
      hspc_error_dialog "Error:  No Sim Files are found.  Please click on 'Edit Sim File' to create one." \
	  $schematic_wt
      return
}

}

proc hspc_do_nothing {dialog} {
   return
}

proc hspc_set_sim_file_name {w} {
    global CUR_SIM_FILE HSPICE_SIM_MENU

    set CUR_SIM_FILE [db::getAttr value -of $w]
#    set menu_attr [db::getAttr title -of $HSPICE_SIM_MENU]
    if {[string compare $CUR_SIM_FILE "test.hspc"] == 0} {
         db::setAttr title -of $HSPICE_SIM_MENU -value "HSPC"
    } else {
         db::setAttr title -of $HSPICE_SIM_MENU -value "HSPC: $CUR_SIM_FILE"
    }
}


# View Log File
proc hspc_view_log_file {schematic_wt} {

global HSPC_HOME CPPSIMSHARED_HOME EDITOR

set title_list [split [db::getAttr title -of $schematic_wt] " "]
set TOP_CELL [lrange $title_list 2 2]
set TOP_LIBRARY [lrange $title_list 1 1]

set cur_dir "$HSPC_HOME/SimRuns"
if {[file isdirectory $cur_dir] == 0} {
      hspc_error_dialog "Error: SimRuns directory does not exist - Click on 'Edit Sim File'" $schematic_wt
      return
}
set cur_dir "$HSPC_HOME/SimRuns/$TOP_LIBRARY"
if {[file isdirectory $cur_dir] == 0} {
      hspc_error_dialog "Error: Lib directory does not exist - Click on 'Edit Sim File'" $schematic_wt
      return
}
set cur_dir "$HSPC_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL"
if {[file isdirectory $cur_dir] == 0} {
      hspc_error_dialog "Error: Module directory does not exist - Click on 'Edit Sim File'" $schematic_wt
      return
}


set CUR_LOG_FILE simrun.lis
set cur_file "$cur_dir/$CUR_LOG_FILE"

####### Open log file
if {[catch "set fid \[open $cur_file r\]" err]} {
   hspc_error_dialog "Error: Hspice log file does not exist - you must run Hspice first" $schematic_wt
   return
}

if {[catch {close $fid} err]} {
   hspc_error_dialog "Error: Hspice log file cannot be closed" $schematic_wt
   return
}

####### Open sim file
if {[catch "exec $EDITOR -bg black -fg yellow -T $cur_file $cur_file &" err]} {
      hspc_error_dialog "Error:  cannot open Log File: $cur_file" $schematic_wt
      return
}

}


proc hspc_error_dialog {message schematic_wt} {

    db::destroy [gi::getDialogs errDialogMessage -parent $schematic_wt]
    set errDialogMessageContainer [gi::createDialog errDialogMessage \
      -title "Error Message" \
      -showHelp false -execProc hspc_do_nothing \
      -parent $schematic_wt]

    set simFileMessage [gi::createLabel errDialogMessageLabel  \
      -parent $errDialogMessageContainer \
      -label "$message" ]
}

proc hspc_operation_success_message_dialog {message schematic_wt} {

    db::destroy [gi::getDialogs errDialogMessage -parent $schematic_wt]
    set errDialogMessageContainer [gi::createDialog errDialogMessage \
      -title "Operation Completed Successfully" \
      -showHelp false -execProc hspc_do_nothing \
      -parent $schematic_wt]

    set simFileMessage [gi::createLabel errDialogMessageLabel  \
      -parent $errDialogMessageContainer \
      -label "$message" ]
}


#########################################################################################################

# Netlist only routine
proc hspc_netlist_only {schematic_wt lite_flag} {


global CPPSIMSHARED_HOME HSPC_HOME CUR_SIM_FILE

set title_list [split [db::getAttr title -of $schematic_wt] " "]
set TOP_CELL [lrange $title_list 2 2]
set TOP_LIBRARY [lrange $title_list 1 1]

puts "--------- running netlister for '$TOP_CELL' (Lib '$TOP_LIBRARY')  --------"

set netlist_dir "$HSPC_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL"

if {[file isdirectory $netlist_dir] == 0} {
   hspc_error_dialog "No Netlist Directory!" $schematic_wt
   puts "--- Netlisting Exited with Errors ---"
   return
}

if {[file isfile "$netlist_dir/netlist.hsp"] != 0} {
    if {[catch {file delete "$netlist_dir/netlist.hsp"} mssg]} {
        hspc_error_dialog "Cannot delete netlist file: $netlist_dir/netlist.hsp" $schematic_wt
        puts "--- Netlisting Exited with Errors ---"
        return
    } 
}

set netlist_obj [nl::getNetlisters HSPICE]
if {$netlist_obj eq ""} {
     hspc_error_dialog "Error: no Hspice netlister found!" $schematic_wt
     puts "--- Netlisting Exited with Errors ---"
     return   
}

db::setAttr netlistEndProc -of $netlist_obj -value "hspc_netlistEndProc"

set cv [dm::findCellView "schematic" -cellName $TOP_CELL -libName $TOP_LIBRARY]
set netlist_result [nl::runNetlister $netlist_obj -cellView $cv -filePath "$netlist_dir/netlist.hsp" \
			-viewSearchList "schematic hspiceD symbol" -viewStopList "hspiceD symbol"]

if {$netlist_result == 0} {
     hspc_error_dialog "Error: Hspice netlisting FAILED!" $schematic_wt
     puts "--- Netlisting Exited with Errors ---"
     return   
} else {
    if {$lite_flag != "lite"} {
        hspc_operation_success_message_dialog "Hspice netlisting completed!" $schematic_wt
    }
     puts "--- Netlisting successfully completed ---"
}

puts "--> Netlist location:  $HSPC_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL/netlist.hsp"

if {$lite_flag != "lite"} {
   if {[file isfile "$HSPC_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL/test.hspc"] == 0} {
       puts "*** Please create a Sim file (click on 'Edit Sim File') ***"
   } else {
       puts "*** Note:  to run Hspice from Linux shell: ***"
       puts "> cd $HSPC_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL"
       puts "> $CPPSIMSHARED_HOME/HspiceToolbox/HSPC/bin/hspc netlist.hsp simrun.sp $CUR_SIM_FILE"
       puts "> hspice -i simrun.sp -o simrun"
       puts "*** Note:  to run Hspice within Matlab: ***"
       puts "> addpath('$CPPSIMSHARED_HOME/HspiceToolbox');"
       puts "> cd $HSPC_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL"
       if {$CUR_SIM_FILE == "test.hspc"} {
          puts "> hspsim_cdesigner_nonet"
       } else {
          puts "> hspsim_designer_nonet('$CUR_SIM_FILE')"
       }
   }
}
}

#########################################################################################################

proc hspc_netlistEndProc {netlist_obj} {
   
set netlist_filename [db::getAttr fileName -of $netlist_obj]
set i [string last "/" $netlist_filename]
set netlist_dir [string range $netlist_filename 0 [expr $i - 1]]
set i [string last "/" $netlist_dir]
set TOP_CELL [string range $netlist_dir [expr $i + 1] end]
puts "Netlist Dir: $netlist_dir"
puts "Top Cell:  $TOP_CELL"

####### Open netlist file
if {[catch "set fid_main \[open $netlist_filename a\]" err]} {
   puts "Error: cannot create netlist file: $netlist_filename"
   return
}

# main module
foreach subdir [glob -directory "$netlist_dir/cnl" -type d *] {
    set i [string last "/" $subdir]
    set subdir_name [string range $subdir [expr $i + 1] end]
    if {[string compare $subdir_name $TOP_CELL] == 0} {
       if {[file exists "$subdir/netlist"] == 1} {
          if {[catch "set fid_sub \[open $subdir/netlist r\]" err]} {
              puts "Error: cannot open sub-netlist file: $subdir/netlist"
              return
          }
          while {[gets $fid_sub line] >= 0} {
              if {[string compare -length 7 $line "* View "] != 0} {
                 puts $fid_main $line
	      }
          }
          if {[catch {close $fid_sub} err]} {
              puts "Error: cannot close sub-netlist file: $subdir/netlist"
              return
          }
       }
    }
}

# sub-modules
foreach subdir [glob -directory "$netlist_dir/cnl" -type d *] {
    set i [string last "/" $subdir]
    set subdir_name [string range $subdir [expr $i + 1] end]
    if {[string compare $subdir_name $TOP_CELL] != 0} {
       if {[file exists "$subdir/netlist"] == 1} {
          if {[catch "set fid_sub \[open $subdir/netlist r\]" err]} {
              puts "Error: cannot open sub-netlist file: $subdir/netlist"
              return
          }
          while {[gets $fid_sub line] >= 0} {
              if {[string compare -length 7 $line "* View "] != 0} {
                 puts $fid_main $line
	      }
          }
          if {[catch {close $fid_sub} err]} {
              puts "Error: cannot close sub-netlist file: $subdir/netlist"
              return
          }
       }
    }
}

puts $fid_main "\n.end"
if {[catch {close $fid_main} err]} {
   puts "Error: cannot close netlist file: $netlist_filename"
   return
}

}


# Netlist and run Hspice routine
proc hspc_netlist_and_run {schematic_wt} {

global CPPSIMSHARED_HOME HSPC_HOME CUR_SIM_FILE HSPICE_RUN_FLAG HSPICE_JOB_ID

if {$HSPICE_RUN_FLAG == 1} {
    hspc_error_dialog "Error: Hspice already running (click on 'Kill Hspice Run' to terminate)" $schematic_wt
    return
}

if {[auto_execok hspice] == ""} {
   hspc_error_dialog "Error: Hspice command not found!" $schematic_wt
   puts "Error:  hspice command not found!"
   puts "      -> make sure that you have installed Hspice and"
   puts "         included it in your path"
   puts "------------------ Error: running hspice failed --------------------"
   return
}

set hspc_command "$CPPSIMSHARED_HOME/HspiceToolbox/HSPC/bin/hspc"
if {[auto_execok $hspc_command] == ""} {
   hspc_error_dialog "Error: HSPC command not found!" $schematic_wt
   puts "Error:  hspc command not found!"
   puts "      -> make sure that you have installed the CppSim framework"
   puts "         hspc assumed location: $hspc_command"
   puts "------------------ Error: running hspc failed --------------------"
   return
}

set title_list [split [db::getAttr title -of $schematic_wt] " "]
set TOP_CELL [lrange $title_list 2 2]
set TOP_LIBRARY [lrange $title_list 1 1]


if {[file isfile "$HSPC_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL/test.hspc"] == 0} {
    hspc_error_dialog "Error: you must create a Sim file (click on 'Edit Sim File')" $schematic_wt
    puts "Error: must create a Sim file (click on 'Edit Sim File')"
    puts "------------------ Error: running hspice failed --------------------"
    return
} 

set HSPICE_RUN_FLAG 1

hspc_netlist_only $schematic_wt "lite"

set netlist_dir "$HSPC_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL"

if {[file isfile "$netlist_dir/netlist.hsp"] == 0} {
   hspc_error_dialog "Error: Hspice netlist file not found!" $schematic_wt
   puts "Error:  Netlist file is missing!"
   puts "      (i.e., can't open file '$netlist_dir/netlist.hsp')"
   puts "------------------ Error: running netlister failed --------------------"
   set HSPICE_RUN_FLAG 0
   return
}

puts "-------------- running HSPC program --------------"

catch {open "|$hspc_command $netlist_dir/netlist.hsp $netlist_dir/simrun.sp \
        $netlist_dir/$CUR_SIM_FILE |& cat"} fid
set message_input [read $fid]
puts $message_input
if {[catch {close $fid} err] != 0 || [file isfile "$netlist_dir/simrun.sp"] == 0} {
    hspc_error_dialog "Error: HSPC command failed!" $schematic_wt
    puts "------------------ HSPC Exited with Errors --------------------"
    set HSPICE_RUN_FLAG 0
    return
}

puts "--- running Hspice (Ouput directory: $HSPC_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL) ---"

set HSPICE_JOB_ID [xt::createJob hspice -type batch -cmdLine "hspice -i $netlist_dir/simrun.sp -o $netlist_dir/simrun" -runDesc "Hspice simulation of module '$TOP_CELL' (lib '$TOP_LIBRARY')" -exitProc hspc_hspice_message]

}


# Read and log output from the program
proc hspc_hspice_message {job_obj} {

global HSPICE_RUN_FLAG

set job_status [db::getAttr exitStatus -of $job_obj]
set run_desc [db::getAttr runDesc -of $job_obj]

if {$job_status == 0} {
   hspc_operation_success_message_dialog "$run_desc successfully completed" [gi::getActiveWindow]
   puts "--- $run_desc successfully completed  ---"
} else {
    if {$HSPICE_RUN_FLAG == 0} {
      hspc_operation_success_message_dialog "Killed $run_desc!" [gi::getActiveWindow]
      puts "*** Killed $run_desc ***"
    } else {
      hspc_error_dialog "Error: $run_desc FAILED! View Log File for Details" [gi::getActiveWindow]
      puts "*** Error: $run_desc FAILED! View Log File for Details ***"
    }
}
set HSPICE_RUN_FLAG 0

# puts "*** finished hspice with status $job_status"
}


# Kill current HspSim run

proc hspc_kill_run {schematic_wt} {

global HSPICE_RUN_FLAG HSPICE_JOB_ID

if {$HSPICE_RUN_FLAG == 1} {
    xt::signalJob $HSPICE_JOB_ID -signal kill
    set HSPICE_RUN_FLAG 0
} else {
    hspc_error_dialog "Error: no Hspice job to kill" $schematic_wt
}

}



proc hspc_start_custom_waveview_tr {schematic_wt} {

global CPPSIMSHARED_HOME HSPC_HOME CUR_SIM_FILE

set title_list [split [db::getAttr title -of $schematic_wt] " "]
set TOP_CELL [lrange $title_list 2 2]
set TOP_LIBRARY [lrange $title_list 1 1]
set run_dir "$HSPC_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL"

if {[file isfile "$run_dir/simrun.tr0"] == 0} {
    hspc_error_dialog "Error: you must run transient sim in Hspice (and probe signals) before starting Custom WaveView (Transient)" $schematic_wt
    puts "Error: you must run transient sim in Hspice (and probe signals) before starting Custom WaveView (Transient)"
    return
} 

xt::createJob custom_waveview_tr -type batch -cmdLine "cd $run_dir; sx -w -k -load simrun.tr0" -runDesc "Custom WaveView (Transient)"

}

proc hspc_start_custom_waveview_ac {schematic_wt} {

global CPPSIMSHARED_HOME HSPC_HOME CUR_SIM_FILE

set title_list [split [db::getAttr title -of $schematic_wt] " "]
set TOP_CELL [lrange $title_list 2 2]
set TOP_LIBRARY [lrange $title_list 1 1]
set run_dir "$HSPC_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL"

if {[file isfile "$run_dir/simrun.ac0"] == 0} {
    hspc_error_dialog "Error: you must run AC sim in Hspice (and probe signals) before starting Custom WaveView (AC)" $schematic_wt
    puts "Error: you must run AC sim in Hspice (and probe signals) before starting Custom WaveView (AC)"
    return
} 

xt::createJob custom_waveview_ac -type batch -cmdLine "cd $run_dir; sx -w -k -load simrun.ac0" -runDesc "Custom WaveView (AC)"

}

proc hspc_start_custom_waveview_dc {schematic_wt} {

global CPPSIMSHARED_HOME HSPC_HOME CUR_SIM_FILE

set title_list [split [db::getAttr title -of $schematic_wt] " "]
set TOP_CELL [lrange $title_list 2 2]
set TOP_LIBRARY [lrange $title_list 1 1]
set run_dir "$HSPC_HOME/SimRuns/$TOP_LIBRARY/$TOP_CELL"

if {[file isfile "$run_dir/simrun.sw0"] == 0} {
    hspc_error_dialog "Error: you must run DC sim in Hspice (and probe signals) before starting Custom WaveView (DC)" $schematic_wt
    puts "Error: you must run DC sim in Hspice (and probe signals) before starting Custom WaveView (DC)"
    return
}

xt::createJob custom_waveview_dc -type batch -cmdLine "cd $run_dir; sx -w -k -load simrun.sw0" -runDesc "Custom WaveView (DC)"

}

